'''
Created on 31. 1. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property

from traitsui.api import \
    View, Item, VGroup, Group

from .reinf_layout_component import \
    ReinfLayoutComponent, \
    STATE_LAW_AND_GEOMETRY_CHANGE, \
    STATE_AND_GEOMETRY_CHANGE

from bmcs_beam.mxn.utils import \
    KeyRef

from bmcs_beam.mxn.material_types import \
    MTReinfBar

class RLCBar(ReinfLayoutComponent):
    '''base class for bar reinforcement
    '''
    def __init__(self, *args, **metadata):
        if not metadata.get('material', None):
            metadata['material'] = 'bar_d10'
        super(RLCBar, self).__init__(**metadata)

    x = Float(0.1, auto_set=False, enter_set=True, geo_input=True)
    z = Float(0.45, auto_set=False, enter_set=True, geo_input=True)
    '''position of the bar with respect to lower left
    corner of reinforced cross section
    '''

    material = KeyRef('bar_d10', db=MTReinfBar.db)

    def convert_eps_u_2_lo(self, eps_up):
        h = self.matrix_cs.geo.height
        eps_u = self.material_law_.eps_u
        eps_lo = eps_up + (eps_u - eps_up) / (h - self.z) * h
        return eps_lo

    def convert_eps_u_2_up(self, eps_lo):
        h = self.matrix_cs.geo.height
        eps_u = self.material_law_.eps_u
        eps_up = eps_lo + (eps_u - eps_lo) / self.z * h
        return eps_up

    eps = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Strain of the bar
    '''
    @cached_property
    def _get_eps(self):
        # ------------------------------------------------------------------------
        # geometric params independent from the value for 'eps_t'
        # ------------------------------------------------------------------------
        height = self.matrix_cs.geo.height
        eps_lo = self.state.eps_lo
        eps_up = self.state.eps_up
        # strain at the height of reinforcement bar [-]:
        #
        return eps_lo + (eps_up - eps_lo) * self.z / height

    sig = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Stress of the bar
    '''
    @cached_property
    def _get_sig(self):
        return self.material_law_.mfn.get_value(self.eps)

    f = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''force in the bar [kN]:
    '''
    @cached_property
    def _get_f(self):
        sig = self.sig
        A_bar = self.material_.area
        return sig * A_bar * self.unit_conversion_factor

    N = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Get the resulting normal force.
    '''
    @cached_property
    def _get_N(self):
        return self.f

    M = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Get the resulting moment.
    '''
    @cached_property
    def _get_M(self):
        height = self.matrix_cs.geo.height
        return self.f * (height - self.z)

    def plot_geometry(self, ax, clr='DarkOrange'):
        '''Plot geometry'''
        ax.plot(self.x, self.z, 'o', color=clr)

    def plot_eps(self, ax):
        ax.hlines([self.z], [0], [-self.eps], lw=4, color='DarkOrange')

    def plot_sig(self, ax):
        h = self.matrix_cs.geo.height
        ax.hlines([self.z], [0], [-self.sig], lw=4, color='DarkOrange')

    node_name = 'Reinforcement Bar'

    tree_view = View(VGroup(
                       Group(
                       Item('x'),
                       Item('z'),
                       label='Position'
                       ),
                       Group(
                       Item('material', show_label=True),
                       Item('material_law', show_label=True),
                       label='Bar material'
                       ),
                       springy=True
                       ),
                       resizable=True,
                       buttons=['OK', 'Cancel']
                       )

if __name__ == '__main__':
    pass



'''
Created on 2. 4. 2014

@author: Vancikv

Standard tree view with the default database node
and added mxn_diagram node
'''
print('x12')

from bmcs_beam.mxn.use_cases import \
    UseCaseContainer
from bmcs_beam.mxn.view import \
    MxNTreeView


ucc = UseCaseContainer()
ucc.use_case_to_add = 'prediction'
ucc.add_use_case = True

uc_view = MxNTreeView(root=ucc)
uc_view.configure_traits()


#!/usr/bin/python
# -*- coding: utf-8 -*-
# Pierre Haessig — March 2014
"""
Qt adaptation of Gael Varoquaux's tutorial to integrate Matplotlib
http://docs.enthought.com/traitsui/tutorials/traits_ui_scientific_app.html#extending-traitsui-adding-a-matplotlib-figure-to-our-application

based on Qt-based code shared by Didrik Pinte, May 2012
http://markmail.org/message/z3hnoqruk56g2bje

adapted and tested to work with PySide from Anaconda in March 2014
"""

from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure
from traits.api import Instance
from traitsui.qt4.basic_editor_factory import BasicEditorFactory
from traitsui.qt4.editor import Editor


# We want matplotlib to use a QT backend
# matplotlib.use('Qt4Agg')


class _MPLFigureEditor(Editor):

    scrollable = True

    def init(self, parent):
        self.control = self._create_canvas(parent)
        self.object.on_trait_change(self.update_editor, 'data_changed')
        self.set_tooltip()

    def update_editor(self):
        figure = self.value
        figure.canvas.mpl_connect('key_press_event', self.key_press_callback)
        figure.canvas.draw()

    def _create_canvas(self, parent):
        """ Create the MPL canvas. """
        # matplotlib commands to create a canvas
        mpl_canvas = FigureCanvas(self.value)
        return mpl_canvas

    def key_press_callback(self, event):
        'whenever a key is pressed'
        figure = self.value
        if not event.inaxes:
            return
        if event.key == 'k':
            if figure.axes[0].get_xscale() == 'log':
                figure.axes[0].set_xscale('linear')
                figure.canvas.draw()
            else:
                figure.axes[0].set_xscale('log')
                figure.canvas.draw()

        if event.key == 'l':
            if figure.axes[0].get_yscale() == 'log':
                figure.axes[0].set_yscale('linear')
                figure.canvas.draw()
            else:
                figure.axes[0].set_yscale('log')
                figure.canvas.draw()


class MPLFigureEditor(BasicEditorFactory):

    klass = _MPLFigureEditor


if __name__ == "__main__":
    # Create a window to demo the editor
    from traits.api import HasTraits, Int, Float, on_trait_change
    from traitsui.api import View, Item
    from numpy import sin, cos, linspace, pi

    class Test(HasTraits):

        figure = Instance(Figure, ())
        n = Int(11)
        a = Float(0.5)

        view = View(Item('figure', editor=MPLFigureEditor(),
                         show_label=False),
                    Item('n'),
                    Item('a'),
                    width=400,
                    height=300,
                    resizable=True)

        def __init__(self):
            super(Test, self).__init__()
            axes = self.figure.add_subplot(111)
            self._t = linspace(0, 2 * pi, 200)
            self.plot()

        @on_trait_change('n,a')
        def plot(self):
            t = self._t
            a = self.a
            n = self.n
            axes = self.figure.axes[0]
            if not axes.lines:
                axes.plot(
                    sin(t) * (1 + a * cos(n * t)), cos(t) * (1 + a * cos(n * t)))
            else:
                l = axes.lines[0]
                l.set_xdata(sin(t) * (1 + a * cos(n * t)))
                l.set_ydata(cos(t) * (1 + a * cos(n * t)))
            canvas = self.figure.canvas
            if canvas is not None:
                canvas.draw()

    t = Test()
    t.configure_traits()



#-------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Apr 15, 2010 by: rch

import fnmatch
from matplotlib.figure import \
    Figure
import os

from traits.api import \
    Str, Enum, Instance, Type, \
    Directory, List, Property, Float, cached_property, \
    Event, on_trait_change
from traitsui.api import \
    View, Item, VGroup, HGroup, HSplit, VSplit, \
    TableEditor, TabularEditor,  \
    Group
from traitsui.table_column import \
    ObjectColumn
from traitsui.table_filter \
    import EvalFilterTemplate, MenuFilterTemplate, RuleFilterTemplate, \
    EvalTableFilter
from traitsui.tabular_adapter \
    import TabularAdapter

from .ex_run import ExRun
from .ex_type import ExType
from matresdev.db.simdb import \
    SimDBClassExt
from matresdev.db.simdb.simdb import \
    simdb
from util.find_class import \
    _find_class
from util.traits.editors.mpl_figure_editor import \
    MPLFigureEditor


class ExRunTableAdapter (TabularAdapter):

    columns = Property

    #---------------------------------------------------------------
    # EXTRACT FACTOR NAMES AS COLUMNS FOR TABLE EDITOR
    #-------------------------------------------------------------------
    def _get_columns(self):
        cols = [('index', 'index')]  # , ('key', 'key') ]
        obj = self.object
        for field_name in obj.field_names:
            cols.append((field_name, field_name))
        print(cols)
        return cols

    selection_mode = 'rows',

    font = 'Courier 10'
    alignment = 'right'
    odd_bg_color = 'lightblue'

    index_width = Float(50)

    index_text = Property

    def _get_index_text(self):
        return str(self.row)

    key_width = Float(120)
    key_text = Property

    def _get_key_text(self):
        factor_idx = self.column - 1
        value = self.object.inst_list[self.row, factor_idx]
        return str(value)

exdb_tabular_editor = TabularEditor(adapter=ExRunTableAdapter(),
                                    multi_select=True,
                                    selected='selected_inst_list',
                                    )

exdb_table_editor = TableEditor(
    columns_name='table_columns',
    selection_mode='rows',
    selected='object.selected_inst_list',
    # selected_indices  = 'object.selected_exruns',
    show_toolbar=True,
    auto_add=False,
    configurable=True,
    sortable=True,
    reorderable=False,
    sort_model=False,
    orientation='vertical',
    auto_size=True,
    filters=[EvalFilterTemplate,
             MenuFilterTemplate,
             RuleFilterTemplate],
    search=EvalTableFilter(),
)

#-------------------------------------------------------------------------
# Class Extension - global persistent container of class inst_list
#-------------------------------------------------------------------------


class ExRunClassExt(SimDBClassExt):

    category = Str('exdata')

    path = List([])

    # dictionary of predefined inst_list - used for
    # debugging and early stages of class developmemnt.
    #
    klass = Type

    classname = Property(depends_on='klass')

    @cached_property
    def _get_classname(self):
        return self.klass.__name__

    field_names = Property(depends_on='klass')

    @cached_property
    def _get_field_names(self):
        '''
        Get the list of table fields.
        '''
        return self.klass.class_trait_names(table_field=lambda x: x != None)

    # Get columns for the table editor
    #
    table_columns = Property(depends_on='klass')

    @cached_property
    def _get_table_columns(self):
        columns = [ObjectColumn(name='key',
                                editable=False,
                                horizontal_alignment='center',
                                )]
        columns += [ObjectColumn(name=field_name,
                                 editable=False,
                                 horizontal_alignment='center',
                                 # width = 100
                                 ) for field_name in self.field_names]
        return columns

    dir = Directory

    def _dir_default(self):
        '''Name of the directory for the data of the class
        '''
        full_path = (simdb.simdb_dir, self.category)
        path = os.path.join(*full_path)
        return path

    def _get_file_list(self):
        '''Populate the inst_list with the values
        '''
        # walk through the directories and read the
        # values
        #
        print('file list', self.dir)
        for root, sub_folders, files in os.walk(self.dir):
            for folder in sub_folders:
                ex_type_file = os.path.join(root, folder, 'ex_type.cls')
                if os.path.exists(ex_type_file):

                    # check if the class specification coincides with
                    # the klass trait of this table
                    #
                    f = open(ex_type_file, 'r')
                    ex_type_klass = f.read().split('\n')[0]  # use trim here
                    f.close()

                    klass = _find_class(ex_type_klass)

                    if klass and klass.__name__ == self.klass.__name__:

                        file_ext = klass().file_ext
                        ex_type_dir = os.path.join(root, folder)
                        ex_type_files = os.listdir(ex_type_dir)
                        for filename in fnmatch.filter(ex_type_files, '*.%s' % file_ext):
                            yield os.path.join(ex_type_dir, filename)

    ex_run_list = List

    def _ex_run_list_default(self):
        print('file_list', self._get_file_list())
        return [ExRun(ex_run_file) for ex_run_file in self._get_file_list()]

    inst_list = List

    def _inst_list_default(self):
        print('ex_run_list', self.ex_run_list)
        return [ex_run.ex_type for ex_run in self.ex_run_list]

    selected_inst_list = List

    def _selected_inst_list_default(self):
        return []

    selected_instance = Property(Instance(ExType),
                                 depends_on='selected_inst_list[]')

    def _get_selected_instance(self):
        if len(self.selected_inst_list) > 0:
            return self.selected_inst_list[0]
        else:
            return None

    def export_inst_list(self):
        ex_table = []
        for inst in self.inst_list:
            row = [getattr(inst, tcol) for tcol in self.table_columns]
            ex_table.append(row)
        print(ex_table)

    def keys(self):
        return list(self.inst_list.keys())

    def get(self, name, Missing):
        it = self.inst_list.get(name, Missing)
        return it

    #-------------------------------------------------------------------
    # PLOT OBJECT
    #-------------------------------------------------------------------

    figure = Instance(Figure)

    def _figure_default(self):
        figure = Figure(facecolor='white')
        # figure.add_axes( [0.08, 0.13, 0.85, 0.74] )
        figure.add_axes([0.15, 0.15, 0.75, 0.75])
        return figure

    # event to trigger the replotting - used by the figure editor
    #
    data_changed = Event

    # selected plot template
    #
    plot_template = Enum(values='plot_template_list')

    # list of availble plot templates
    # (gets extracted from the model whenever it's been changed)
    #
    plot_template_list = Property(depends_on='klass')

    @cached_property
    def _get_plot_template_list(self):
        '''Change the selection list of plot templates.

        This method is called upon every change of the model. This makes the viewing of
        different experiment types possible.
        '''
        return list(self.inst_list[0].plot_templates.keys())

    @on_trait_change('selected_inst_list,plot_template')
    def redraw(self, ui_info=None):
        ''' Use the currently selected plot template to plot it in the Figure.
        '''

        # map the array dimensions to the plot axes
        #
        figure = self.figure

        axes = figure.gca()
        axes.clear()

        # get the labels (keys) of the selected inst_list for
        # attibution of the legends in matplotlib:
        #

        for run in self.selected_inst_list:
            proc_name = run.plot_templates[self.plot_template]
            plot_processor = getattr(run, proc_name)
            plot_processor(axes)

        legend_names = []
        for run in self.selected_inst_list:
            legend_names = legend_names + [run.key]
        if len(legend_names) > 0:
            axes.legend(legend_names, loc=7)

        self.data_changed = True

    #-------------------------------------------------------------------------
    # VIEW
    #-------------------------------------------------------------------------

    traits_view = View(
        HSplit(
            VSplit(
                VGroup(
                    HGroup(Item('classname',
                                emphasized=True,
                                style='readonly',
                                label='database extension class')
                           ),
                    Item('inst_list',
                         editor=exdb_table_editor,
                         show_label=False,
                         style='custom',
                         resizable=True),
                ),
                label='experiment table',
                id='exdb.table.inst_list',
                dock='tab',
                scrollable=True,
            ),
            VGroup(
                VGroup(
                    Item('selected_instance@',
                         resizable=True,
                         show_label=False),
                    label='experiment',
                    id='exdb.table.instance',
                    dock='tab',
                    scrollable=True,
                ),
                Group(
                    Item('figure', editor=MPLFigureEditor(),
                         resizable=True, show_label=False),
                    id='exrun_table.plot_sheet',
                    label='plot sheet',
                    dock='tab',
                ),
                Group(
                    Item('plot_template'),
                    columns=1,
                    label='plot parameters',
                    id='exrun_table.plot_params',
                    dock='tab',
                ),
                id='exdb.table.right',
                layout='split',
                label='selected instance',
                dock='tab',
            ),
            id='exdb.table.splitter',
        ),
        id='exdb.table',
        buttons=['OK', 'Cancel'],
        dock='tab',
        resizable=True,
        height=0.8, width=0.8,
    )



'''
Created on 1. 7. 2014

@author: Vancikv
'''

from traits.api import \
    HasStrictTraits, List, Str, Button, \
    Trait, Instance

from traitsui.api import \
    View, Item, VGroup, HGroup, UItem, spring

from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode

from .use_case_parametric_study import \
    UCParametricStudy

from bmcs_beam.mxn.mxn_diagram import \
    MxNDiagram

from bmcs_beam.mxn.ecb_calib import \
    ECBCalib

from .use_case_database import \
    UCDatabase

class UseCaseContainer(MxNTreeNode):
    '''Contains MxN diagrams wrapped in MxNDescription classes.
    MxNDescription controls plotting of particular MxNDiagram
    that can added/removed or modified ,e.g. via tree editor.
    '''

    node_name = Str('Use cases')

    use_case_to_add = Trait('pstudy', {'pstudy'      :   UCParametricStudy,
                                       'calibration' :   ECBCalib,
                                       'prediction'   :   MxNDiagram}
                            )

    add_use_case = Button('Add')
    def _add_use_case_fired(self):
        self.tree_node_list.append(self.use_case_to_add_())

    tree_view = View(HGroup(UItem('use_case_to_add', springy=True),
                            UItem('add_use_case')),
                            spring)

    tree_node_list = List(Instance(MxNTreeNode))
    def _tree_node_list_default(self):
        return [UCDatabase()]



'''
Created on Apr 28, 2010

@author: alexander
'''
from traits.api import \
    HasTraits, Float, Property, cached_property, \
    Instance, File, List, on_trait_change, Int, Tuple, Bool, \
    DelegatesTo, Event, Str, Button, Dict, Array, Any, Enum, Callable
    
from traitsui.api import \
    View, Item, Tabbed, VGroup, HGroup, ModelView, HSplit, VSplit, \
    CheckListEditor, EnumEditor, TableEditor, TabularEditor, Handler, \
    Group, CancelButton, FileEditor

from etsproxy.traits.ui.menu import \
    Action, CloseAction, HelpAction, Menu, \
    MenuBar, NoButtons, Separator, ToolBar                    

from etsproxy.traits.ui.key_bindings import \
    KeyBinding, KeyBindings

from etsproxy.traits.ui.tabular_adapter \
    import TabularAdapter
    
from etsproxy.pyface.api import \
    ImageResource, confirm, error, information, warning, YES, NO, CANCEL
    
from etsproxy.traits.ui.menu import \
    OKButton

from numpy import \
    array, linspace, frompyfunc, zeros, column_stack, \
    log as ln, append, logspace, hstack, sign, trapz, mgrid, c_, \
    zeros
                    
from math import \
    exp, e, sqrt, log, pi
    
from .ex_run import \
    ExRun

from .ex_run_view import \
    ExRunView

from matresdev.db.simdb import \
    SimDB

import os

from matresdev.db.matdb.trc.concrete_mixture \
    import ConcreteMixture

from matresdev.db.matdb.trc.fabric_layout \
    import FabricLayOut

from matresdev.db.matdb.trc.fabric_layup \
    import FabricLayUp

from matresdev.db.matdb.trc.composite_cross_section \
    import CompositeCrossSection, plain_concrete

if __name__ == '__main__':
    # Access to the toplevel directory of the database
    #
    simdb = SimDB()
    
    #--------------------------------------------------------------------------------
    # run a test cycle of loading - saving - loading
    #--------------------------------------------------------------------------------
    
    #----------------------------------------------------------
    # define the path to a tensile test:
    #----------------------------------------------------------
    # - two tests have the same cross section
    # - one test has a different cross section and different layup
    
    ex_path_TT_7a_V1 = os.path.join(simdb.exdata_dir, 'tensile_tests', 'TT-7a',
                                'TT08-7a-V1.DAT')
    
    ex_path_TT_7a_V2 = os.path.join(simdb.exdata_dir, 'tensile_tests', 'TT-7a',
                                'TT08-7a-V2.DAT')
    
    ex_path_TT_9u = os.path.join(simdb.exdata_dir, 'tensile_tests', 'TT-9u',
                                'TT06-9u-V2all90.DAT')
    
    # define the path to a plate test:
    ex_path_PT_10a = os.path.join(simdb.exdata_dir, 'plate_tests', 'PT-10a',
                                'PT10-10a.DAT')
    
    #----------------------------------------------------------
    # delete the pickle file if it exists:
    #----------------------------------------------------------
    print('XXX Delete the pickle file if it exists--------------------')
    data_file = ex_path_TT_7a_V1
    dir_path = os.path.dirname(data_file)
    file_name = os.path.basename(data_file)
    file_split = file_name.split('.')
    pickle_file_name = os.path.join(dir_path, file_split[0] + '.pickle') 
    
    if os.path.exists(pickle_file_name):
        print('--- pickle file removed: ', pickle_file_name, ' ---')
        os.remove(pickle_file_name)
    else:
        print('--- pickle file does not exist: ', pickle_file_name, ' ---')
        
    #----------------------------------------------------------
    # construct ExRunView and show attributes
    #----------------------------------------------------------
    print('XXX Construct ExRunView and show attributes--------------------')
    exrv = ExRunView(data_file = ex_path_TT_7a_V1)
    print('--- GET: data_file = ', exrv.data_file, '---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    print('--- GET: ccs.flu_list[0].s_tex_z = ', exrv.model.ex_type.ccs.fabric_layup_list[0].s_tex_z, '---')
    print('--- GET: ccs.cm_key = ', exrv.model.ex_type.ccs.concrete_mixture_key, '---')
    
    #----------------------------------------------------------
    # save default settings in pickle file
    #----------------------------------------------------------
    print('XXX save default settings in pickle file --------------------')
    exrv.save_run()
    print('--- SAVE RUN ---')
    print('--- pickle file saved to:', pickle_file_name, ' ---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    
    #----------------------------------------------------------
    # change attribute in 'ccs' and save changes in pickle file
    #----------------------------------------------------------
    print('XXX Change attribute in ccs and save changes in pickle file--------------------')
    exrv.model.ex_type.ccs.concrete_mixture_key = 'PZ-0708-1'
    print('--- SET: ccs.cm_key = ', exrv.model.ex_type.ccs.concrete_mixture_key, '---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    exrv.save_run()
    print('--- SAVE RUN ---')
    print('--- pickle file saved to:', pickle_file_name, ' ---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    
    #----------------------------------------------------------
    # change data file to another file and then change it back 
    #----------------------------------------------------------
    print('XXX Change data file to another file and then change it back--------------------')
    print('--- GET: data file (old): ', exrv.data_file)
    exrv.data_file = ex_path_TT_7a_V2
    print('--- SET: data file (new): ', exrv.data_file)
    print('--- GET: unsaved (new) = ', exrv.unsaved, '---')
    if exrv.unsaved == True:
        exrv.save_run()
        print('--- SAVE RUN (new) ---')
    exrv.data_file = ex_path_TT_7a_V1
    print('--- SET: data file (back to old): ', exrv.data_file)
    
    #----------------------------------------------------------
    # show attributes after reloading 
    #----------------------------------------------------------
    print('XXX Show attribute after reloading --------------------')
    print('--- GET: data_file = ', exrv.data_file, '---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    print('--- GET: ccs.flu_list[0].s_tex_z = ', exrv.model.ex_type.ccs.fabric_layup_list[0].s_tex_z, '---')
    print('--- GET: ccs.cm_key = ', exrv.model.ex_type.ccs.concrete_mixture_key, '---')
    
    #----------------------------------------------------------
    # change attribute in 'ccs' 
    #----------------------------------------------------------
    print('XXX Change attribute in ccs --------------------')
    exrv.model.ex_type.ccs.concrete_mixture_key = 'FIL-10-09'
    print('--- SET: ccs.cm_key = ', exrv.model.ex_type.ccs.concrete_mixture_key, '---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    exrv.save_run()
    print('--- SAVE RUN ---')
    print('--- pickle file saved to:', pickle_file_name, ' ---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    
    #----------------------------------------------------------
    # change data file to another file and then change it back 
    #----------------------------------------------------------
    print('XXX Change data file to another file and then change it back--------------------')
    exrv.data_file = ex_path_TT_7a_V2
    print('--- SET: data file (new): ', exrv.data_file)
    exrv.data_file = ex_path_TT_7a_V1
    print('--- SET: data file (back to old): ', exrv.data_file)
    
    #----------------------------------------------------------
    # change attribute in 'ccs.fabric_layup_list' 
    #----------------------------------------------------------
    print('XXX change attribute in ccs.fabric_layup_list --------------------')
    exrv.model.ex_type.ccs.fabric_layup_list.append(plain_concrete(0.01))
    print('--- SET: ccs.fabric_layup_list = ', exrv.model.ex_type.ccs.fabric_layup_list, '---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    exrv.save_run()
    
    #----------------------------------------------------------
    # change data file to another file and then change it back 
    #----------------------------------------------------------
    print('XXX Change data file to another file and then change it back--------------------')
    exrv.data_file = ex_path_TT_7a_V2
    print('--- SET: data file (new): ', exrv.data_file)
    exrv.data_file = ex_path_TT_7a_V1
    print('--- SET: data file (back to old): ', exrv.data_file)
    
    #----------------------------------------------------------
    # change attribute in 'ccs.fabric_layup_list[0].s_tex_z' 
    #----------------------------------------------------------
    print('XXX change attribute in ccs.fabric_layup_list[0].s_tex_z --------------------')
    exrv.model.ex_type.ccs.fabric_layup_list[0].s_tex_z = 0.02 
    print('--- SET: ccs.fabric_layup_list[0].s_tex_z = ', exrv.model.ex_type.ccs.fabric_layup_list[0].s_tex_z, '---')
    print('--- GET: unsaved = ', exrv.unsaved, '---')
    exrv.save_run()



# Import Standard Libraries
import logging
# import scipy as np
import matplotlib.pyplot as plt

class TeeBeamPlot:
    bf = 1200
    hf = 150

    bw = 300
    h = 1200
    d_P = 957
    e_P = 43
    d_S = 1000

    Ap = 2400
    As = 0

    fck = 30
    fsk = 500
    fpk = 1860

    Ep = 195000
    Pm = 2772000
    units = "MPa"

    def __init__(self):
        self.sigma_p = self.Pm/self.Ap
        self.hw = self.h-self.hf
        Af = self.hf*self.bf
        Aw = self.bw*self.hw

        self.Ac = Af + Aw
        self.na = (Af*(self.hw+self.hf/2)+Aw*self.hw/2)/self.Ac
        self.I = self.bw*self.hw**3/12 + self.bf*self.hf**3/12 \
            + Af*(self.h-self.na-self.hf/2)**2 + Aw*(self.hw/2-self.na)
        logging.debug("    Ac = {:6.2f}, I = {:10.2f}".format(self.Ac, self.I))
        self.ta = self.h/100

    def plotTeeBeam(self):
        x0 = self.bf
        x1 = (self.bf + self.bw)/2
        x2 = (self.bf - self.bw)/2

        y0 = self.h
        y1 = self.h - self.hf
        X = [0, x0, x0, x1, x1, x2, x2, 0, 0]
        Y = [y0, y0, y1, y1, 0, 0, y1, y1, y0]
                
        self.fig = plt.figure(figsize=(9, 3))
        ax = self.fig.add_subplot(131)
        ax.plot(X, Y, linewidth=1, color='k')
        ax.fill_between(X, Y,alpha=0.5)

        # ax.plot([0, self.bf],[self.na, self.na], linestyle='--',color='k')

        ax.plot(self.bf/2, self.na, marker = '+', color='k')

        if self.Ap != 0:
            ax.plot(self.bf/2, self.h-self.d_P, marker = 'o', color='r')

        if self.As != 0:
            ax.plot(self.bf/2, self.h-self.d_S, marker = 'o', color='k')

        ax.plot(self.bf/2, self.na+self.e_P, marker='x', color='k')
        # plt.show()

    def plotStressDist(self,Xu,*args):

        ax = self.fig.add_subplot(132)
        ax.plot([0,0], [0,self.h], linewidth=2, color='k')

        # Moment Capacity
        if Xu < self.hf:
            Nc1 = args[0]
            print(Nc1)
        elif Xu > self.hf:


            [fcd, Nc1,y1,Nc2,y2,Ns,y3] = args

            h0 = self.h - self.hf
            self.__plotConcreteStress(ax,fcd,self.h,h0)

            h1 = self.h - Xu
            self.__plotConcreteStress(ax, fcd, self.h, h1)

            self.__plotSteelStress(ax, Ns, y3)
            print(Ns)
            # y1 = [self.h, self.h, h1, h1, self.h]


            h1 = h1 + Xu/2

            plt.xlim((-1.2*fcd,1.2*fcd))
            plt.show()

    # Private:

    def __plotConcreteStress(self,ax,fcd,h_top,h_bot):
        X0 = [-fcd, 0]
        Y0 = [h_top, h_top]
        Y1 = [h_bot, h_bot]
        ax.fill_between(X0, Y0, Y1, color='b', alpha=0.3)
        ax.arrow(fcd, (h_top+h_bot)/2, -9*fcd/10, 0,
        width=self.ta,  fc='b', ec='b', head_width=3*self.ta, head_length=self.ta/10)
    
    def __plotSteelStress(self,ax,Ns,h):
        if Ns > 0:
            ax.arrow(0, h, 9*Ns/10, 0,
            width=self.ta,  fc='r', ec='r', head_width=3*self.ta, head_length=self.ta/10)

if __name__ == "__main__":
    beam = TeeBeamPlot()
    beam.plotTeeBeam()
    Xu = 224
    fcd = 20
    Nc1 = 27
    y1 = 282
    Nc2 = 107
    y2 = 267
    Ns = 2
    y3 = 18

    beam.plotStressDist(Xu,fcd,Nc1,y1,Nc2,y2,Ns,y3)




# Import Standard Libraries
from abc import ABCMeta, abstractmethod
import logging
import scipy as np

# Import Local Libraries
import Util_ACI as ACI
import Util_EC2 as EC2


#===========================================================================
#   Reinforced Beam
#===========================================================================

class ReinforcedBeam(metaclass=ABCMeta):
	""" Abstract Reinforced Beam Class
	
	Static Members:
		gamma_c = 1.5
		gamma_S = 1.15
		niter = 20

	Instance Members:
	
	Static Methods:
		plotArrow(self, ax, x, y, dx, dy, color)
		plotStress(self, ax, fcd, h_top, h_bot)
	
	"""

	gamma_c = 1.5
	gamma_S = 1.15
	niter = 20

	H = d_S = As = fck = fyk = Es = k = 0
	units = "MPa"

	def __init__(self, name=None):
		self.ta = self.H/500
		self.name = name

	def plotArrow(self, ax, x, y, dx, dy, color):
		if x == 0:
			ax.arrow(x, y, dx - self.ta, dy, fc=color, ec=color,
                            width=10*self.ta, head_width=10*self.ta, head_length=self.ta)
		else:
			ax.arrow(x, y, dx + self.ta, dy, fc=color, ec=color,
                            width=self.ta, head_width=10*self.ta, head_length=self.ta)

	@staticmethod
	def plotStress(ax, fcd, h_top, h_bot):
		X0 = [-fcd, 0]
		Y0 = [h_top, h_top]
		Y1 = [h_bot, h_bot]
		ax.fill_between(X0, Y0, Y1, color='b', alpha=0.3)

	@staticmethod
	def neutralAxis(h, bb, bt):
		""" Input:	h = trapezoidal height
					bb = bottom width
					bt = top width
			Output:	yc = bottom to neutral axis """
		return h*(2*bt + bb)/3/(bb + bt)

	@staticmethod
	def momentInertia(h, bb, bt):
		""" Input:	h = trapezoidal height
					bb = bottom width
					bt = top width
			Output:	I = moment of inertia about neutral axis """
		return h**3*(bt**2+4*bt*bb+bb**2)/36/(bt+bb)

	@staticmethod
	def beamFactory():
		pass

#===========================================================================
#   Rectangular Beam
#===========================================================================


class RectangularBeam(ReinforcedBeam):
	# b = 558.8
	# h = 609.6
	# d = 546.1
	# As = 3870
	# fck = 27.6
	# fyk = 414
	# rho = 25
	# Es = 200000
	# units = "MPa"
	b = 22
	h = 24
	d = 21.5
	As = 6
	fck = 4000
	fyk = 60000
	rho = 145
	Es = 29000000
	units = "psi"

	#---------------------------------------------------------------------------
	#   ACI Equations
	#---------------------------------------------------------------------------

	def ACI_cracking_moment(self):

		logging.debug("Uncracked moment capacity per ACI")
		Ec = ACI.elastic_modulus(self.fck, self.rho, self.units)
		fr = ACI.tensile_strength(self.fck, self.units)
		n = self.Es/Ec
		logging.debug("    fr = {:6.2f}, Es/Ec = {:3.2f}".format(fr, n))

		Ac = self.b*self.h
		As = (n-1)*self.As
		logging.debug("    b*h = {:6.2f}, (n-1)*As = {:6.2f}".format(Ac, As))

		y_bott = (Ac*self.h/2 + As*(self.h-self.d))/(Ac+As)
		I_uncr = self.b*self.h**3/12\
			+ Ac*(self.h/2-y_bott)**2\
			+ As*(y_bott-(self.h-self.d))**2
		logging.debug("    Botton to NA: y_bott = {:6.2f}".format(y_bott))
		logging.debug("    Moment of inertia = {:10.2f} ".format(I_uncr))

		Mcr = fr*I_uncr/y_bott
		logging.info("    Mcr = {:5.2f}".format(Mcr))
		return Mcr

	def ACI_elastic_moment(self):

		Ec = ACI.elastic_modulus(self.fck, self.rho, self.units)
		fc = 0.5*self.fck
		n = self.Es/Ec
		As = n*self.As
		fs = self.fyk/n
		logging.debug("    n = {:3.2f}, n*As = {:6.2f}".format(n, As))

		kd = (-As+np.sqrt(As**2+2*self.b*As*self.d))/(self.b)
		Icr = self.b*kd**3/12 + self.b*kd*(kd/2)**2 + As*(self.d-kd)**2
		logging.debug("    Top to NA: kd = {:6.2f}".format(kd))
		logging.debug("    Moment of inertia = {:10.2f}".format(Icr))

		Mel = min(fc*Icr/kd, fs*Icr/(self.d-kd))
		logging.info("    Mel = {:5.2f}".format(Mel))
		return Mel

	def ACI_design_moment(self):

		beta1 = ACI.beta(self.fck, self.units)
		logging.debug("    beta1 = {:3.2f}".format(beta1))
		c = (self.As*self.fyk)/(0.85*self.fck*self.b)/beta1
		logging.debug("    c = {:6.2f}".format(c))
		phi = ACI.ductility_requirement(c, self.d, type="beam")
		logging.debug("    phi = {:3.2f}".format(phi))
		ACI.steel_ratio(self.As, self.fck, self.fyk,
                  self.b, self.d, self.units)
		MRd = phi*self.As*self.fyk*(self.d-beta1*c/2)
		logging.info("    MRd = {:5.2f}".format(MRd))
		return MRd

	#---------------------------------------------------------------------------
	#   EC2 Equations
	#---------------------------------------------------------------------------

	def EC2_cracking_moment(self):

		logging.debug("Uncracked moment capacity per EC2")
		Ec = EC2.elastic_modulus(self.fck, self.units)
		fr = EC2.flex_tensile_strength(self.fck, self.h, self.units)
		n = self.Es/Ec
		logging.debug("    fr = {:6.2f}, Es/Ec = {:3.2f}".format(fr, n))

		Ac = self.b*self.h
		As = (n-1)*self.As
		logging.debug("    b*h = {:6.2f}, (n-1)*As = {:6.2f}".format(Ac, As))

		y_bott = (Ac*self.h/2 + As*(self.h-self.d))/(Ac+As)
		I_uncr = self.b*self.h**3/12\
			+ Ac*(self.h/2-y_bott)**2\
			+ As*(y_bott-(self.h-self.d))**2
		logging.debug("    Botton to NA: y_bott = {:6.2f}".format(y_bott))
		logging.debug("    Moment of inertia = {:10.2f} ".format(I_uncr))

		Mcr = fr*I_uncr/y_bott
		logging.info("    Mcr = {:5.2f}".format(Mcr))
		return Mcr

	def EC2_elastic_moment(self):

		Ec = EC2.elastic_modulus(self.fck, self.units)
		fc = 0.5*self.fck
		n = self.Es/Ec
		As = n*self.As
		fs = self.fyk/n
		logging.debug("    n = {:3.2f}, n*As = {:6.2f}".format(n, As))

		kd = (-As+np.sqrt(As**2+2*self.b*As*self.d))/(self.b)
		Icr = self.b*kd**3/12 + self.b*kd*(kd/2)**2 + As*(self.d-kd)**2
		logging.debug("    Top to NA: kd = {:6.2f}".format(kd))
		logging.debug("    Moment of inertia = {:10.2f}".format(Icr))

		Mel = min(fc*Icr/kd, fs*Icr/(self.d-kd))
		logging.info("    Mel = {:5.2f}".format(Mel))
		return Mel

	def EC2_design_moment(self):
		[alpha, beta] = EC2.alpha_beta(self.fck, self.units)
		logging.debug("    a = {:3.2f}, b = {:3.2f}".format(alpha, beta))
		fcd = self.fck/self.gamma_c
		fyd = self.fyk/self.gamma_S
		logging.debug("    fcd = {:6.2f}, fyd = {:6.2f}".format(fcd, fyd))
		Xu = self.As*fyd/(alpha*self.b*fcd)
		Xu_max = EC2.ductility_requirement(
			Xu, self.d, self.fck, fyd, self.units)

		EC2.steel_ratio(self.As, self.fck, self.fyk, self.b,
                  self.d, self.h, Xu_max, self.units)

		MRd = self.As*fyd*(self.d-beta*Xu)
		logging.info("    MRd = {:5.2f}".format(MRd))
		return MRd


class DoublyReinforcedBeam():
	a = 2
	h = 609.6
	d_1 = 20
	As_1 = 10
	d_2 = 546.1
	As_2 = 40
	b = 558.8
	As = 2.8575**2*np.pi/4
	fck = 27.6
	fyk = 414
	rho = 25
	Es = 200000
	units = "MPa"


if __name__ == "__main__":
	logging.basicConfig(level=logging.DEBUG, format='%(message)s')

	# print("\n Ec for C30/37: \n")
	# print(ACI.elastic_modulus(30, rho=25, units="MPa"))
	# print(EC2.elastic_modulus(30))
	# print("\n Tensile strength for C30/37: \n")
	# print(ACI.tensile_strength(30, units="MPa"))
	# print(EC2.tensile_strength(30))
	# print("\n Tensile strength for C55/67: \n")
	# print(ACI.tensile_strength(55, units="MPa"))
	# print(EC2.tensile_strength(55))
	# print("\n Ultimate strain for C30/37 \n")
	# print(ACI.ultimate_strain(30, units="MPa"))
	# print(EC2.ultimate_strain(30))
	# print("\n Ultimate strain for C55/67 \n")
	# print(ACI.ultimate_strain(55, units="MPa"))
	# print(EC2.ultimate_strain(55))
	# print("\n alpha & Beta factors: \n")
	# print(EC2.alpha_beta(10))
	# print(EC2.alpha_beta(60))
	# print(EC2.alpha_beta(80))
	# print("\n Lambda & Eta factors: \n")
	# print(EC2.lambda_eta(10))
	# print(EC2.lambda_eta(60))
	# print(EC2.lambda_eta(80))

	beam = RectangularBeam()
	# print('\n Cracking Moment: \n')
	# MRd = beam.ACI.cracking_moment()
	# print(MRd/12000)
	# MRd = beam.EC2.cracking_moment()
	# print(MRd/12000)

	print('\n Elastic Moment: \n')
	MRd = beam.ACI_elastic_moment()
	print(MRd/12000)
	MRd = beam.EC2_elastic_moment()
	print(MRd/12000)

	print('\n Design Moment: \n')
	MRd = beam.ACI_design_moment()
	print(MRd/12000)
	MRd = beam.EC2_design_moment()
	print(MRd/12000)



# -------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Feb 15, 2010 by: rch

# @todo: introduce the activation of filters - ironing, smoothing

import configparser
import importlib

import os

from string import split
import string
import sys
from traits.api import \
    File, \
    Array, Str, Property, cached_property, \
    Dict, Bool, implements, Float, Callable, List
import zipfile

from .i_ex_type import \
    IExType
from .loadtxt_novalue import loadtxt_novalue
from matresdev.db import SimDBClass
from matresdev.db.simdb import SFTPServer
from matresdev.db.simdb.simdb import simdb
import numpy as np


def comma2dot(c):
    '''convert float with comma separator into float with dot separator'''
    return float((str(c)).replace(",", "."))


def time2sec(date):
    '''convert time format (hh:mm:ss) to seconds (s)'''
    d_list = str(date).split()
    t_list = d_list[1].split(':')
    t_sec = int(t_list[0]) * 60 * 60 + int(t_list[1]) * \
        60 + float(comma2dot(t_list[2]))
    return t_sec


def scaledown_data(data_arr, n_avg):
    '''scaledown the nuber of rows in 'data_array' by the
    integer 'n_avg', i.e. if 'n_avg=2' reduce the number of rows in
    'data_arr' to half. The methods cuts of  up to 2*'n_avgs' rows from the end
    of the file in order to make sure that the sub arrays used for averaging have the same
    shape; in general that doesn't effect the data as the measuring is continued after rupture
    or high frequency measurements is used'''
    n_rows = data_arr.shape[0]
    n_steps = (n_rows - n_avg) / n_avg
    n_max = n_steps * n_avg
    avg_list = [data_arr[i:n_max:n_avg, :] for i in range(n_avg)]
    avg_arr = np.array(avg_list)
    return np.mean(avg_arr, 0)


class ExType(SimDBClass):

    '''Read the data from the directory
    '''

    implements(IExType)

    data_file = File

    file_ext = Str('DAT')

    def validate(self):
        '''Validate the input data return the info whether or not
         the input is valid. This is the condition for processing
         of the derived data.
        '''
        return True

    # set a flag for the view to check whether derived data is available
    #
    derived_data_available = Bool(False)

    # specify inputs
    #
    key = Property(Str, trantient=True, depends_on='data_file')

    def _get_key(self):
        return split(os.path.basename(self.data_file), '.')[0]

    def _set_key(self, value):
        genkey = split(os.path.basename(self.data_file), '.')[0]
        if genkey != value:
            raise KeyError('key mismatch %s != %s' % (genkey, value))

    def __setstate__(self, state, kw={}):
        if 'key' in state:
            del state['key']
        super(SimDBClass, self).__setstate__(state, **kw)

    # indicate whether the test is suitable and prepared for
    # calibration.
    ready_for_calibration = Property(Bool)

    def _get_ready_for_calibration(self):
        # return False by default
        # the subclasses shall overload this
        # and define the rules
        return False

    # specify plot templates that can be chosen for viewing
    #
    plot_templates = Dict(transient=True)

    # define processing
    #
    processed_data_array = Array('float_', transient=True)

    def process_source_data(self):
        '''process the source data and assign
        attributes to the DAT-file channel names.
        '''
        print('*** process data ***')
        self._import_processor()
        self._apply_data_reader()
        self.processed_data_array = self.data_array
        self._set_array_attribs()
        self._apply_data_processor()

    data_columns = Callable(None)
    '''Specification of the measured data columns in the data array 
    '''
    data_units = Callable(None)
    '''Specification of the measured data units in the data array 
    '''
    data_reader = Callable(None)
    '''Function reading the data into the self.data_array
    '''
    data_processor = Callable(None)
    '''Function preparing the data for evaluation - convert the measured
    data to a standard format, smooth the data, remove jumps etc. 
    '''

    def _import_processor(self):
        '''Check to see if there is a data processor in the data directory.
        The name of the processor is assumed data_processor.py.

        '''
        dp_file = os.path.join(os.path.dirname(self.data_file),
                               'data_processor.py')
        dp_modpath = os.path.join(os.path.dirname(self.data_file),
                                  'data_processor').replace(simdb.pathchar, '.')[1:]
        exdata_dir = simdb.exdata_dir
        print('dp_modpath', dp_modpath)
        print('exdata_dir', exdata_dir)
        dp_mod = dp_modpath[len(exdata_dir):]
        print('dp_mod', dp_mod)
        print('sys.path', sys.path)
        if os.path.exists(dp_file):
            mod = importlib.import_module(dp_mod)
            print('simdb-data processor used')
            if hasattr(mod, 'data_columns'):
                self.data_columns = mod.data_columns
            if hasattr(mod, 'data_units'):
                self.data_units = mod.data_units
            if hasattr(mod, 'data_processor'):
                self.data_processor = mod.data_processor
            if hasattr(mod, 'data_reader'):
                self.data_reader = mod.data_reader

    processing_done = Bool(False)

    def _apply_data_processor(self):
        '''Make a call to a test-specific data processor
        transforming the data_array to standard response variables.
        of the test setup.

        An example of data processing for a tensile test is the calculation
        of average displacement from several gauges placed on different sides
        of the specimen. 
        '''
        if self.data_processor:
            self.data_processor(self)
            self.processing_done = True

    def _apply_data_reader(self):
        if self.data_reader:
            self.data_reader(self)
        else:
            self._read_data_array()

    data_array = Array(float, transient=True)

    unit_list = Property(depends_on='data_file')

    def _get_unit_list(self):
        if self.data_units:
            return self.data_units(self)
        else:
            return self.names_and_units[1]

    factor_list = Property(depends_on='data_file')

    def _get_factor_list(self):
        if self.data_columns:
            return self.data_columns(self)
        else:
            return self.names_and_units[0]

    names_and_units = Property(depends_on='data_file')

    @cached_property
    def _get_names_and_units(self):
        ''' Extract the names and units of the measured data.
        The order of the names in the .DAT-file corresponds
        to the order of the .ASC-file.
        '''
        # for data exported into DAT and ASC-files
        file_ = open(self.data_file, 'r')
        lines = file_.read().split()
        names = []
        units = []
        for i in range(len(lines)):
            if lines[i] == '#BEGINCHANNELHEADER':
                print('names and units are defined in DAT-file')
                name = lines[i + 1].split(',')[1]
                unit = lines[i + 3].split(',')[1]
                names.append(name)
                units.append(unit)

        # for data exported into a single csv-file
        file_split = self.data_file.split('.')
        if os.path.exists(file_split[0] + '.csv'):
            file_ = open(file_split[0] + '.csv', 'r')
            header_line_1 = file_.readline().strip()
            if header_line_1.split(';')[0] == 'Datum/Uhrzeit':
                print('csv-file with header exists')
                header_line_2 = file_.readline().strip()
                names = header_line_1.split(';')
                units = header_line_2.split(';')
                names[0] = 'Bezugskanal'
                units[0] = 'sec'

        return names, units

    def _names_and_units_default(self):
        ''' Extract the names and units of the measured data.
        The order of the names in the .DAT-file corresponds
        to the order of the .ASC-file.
        '''
        # for data exported into DAT and ASC-files
        file_ = open(self.data_file, 'r')
        lines = file_.read().split()
        names = []
        units = []
        for i in range(len(lines)):
            if lines[i] == '#BEGINCHANNELHEADER':
                name = lines[i + 1].split(',')[1]
                unit = lines[i + 3].split(',')[1]
                names.append(name)
                units.append(unit)

        # for data exported into a single csv-file
        file_split = self.data_file.split('.')
        if os.path.exists(file_split[0] + '.csv'):
            file_ = open(file_split[0] + '.csv', 'r')
            header_line_1 = file_.readline()
            if header_line_1.split(';')[0] == 'Datum/Uhrzeit':
                header_line_2 = file_.readline()
                names = header_line_1.split(';')
                units = header_line_2.split(';')
                names[0] = 'Bezugskanal'
                units[0] = 'sec'
                # cut off trailing '\r\n' at end of header line
                names[-1] = names[-1][:-2]
                units[-1] = units[-1][:-2]

        print('names, units (default)', names, units)
        return names, units

    def _set_array_attribs(self):
        '''Set the measured data as named attributes defining slices into
        the processed data array.
        '''
        for i, factor in enumerate(self.factor_list):
            self.add_trait(
                factor, Array(value=self.processed_data_array[:, i],
                              transient=True))

    # ------------------

    def _read_data_array(self):
        ''' Read the experiment data.
        '''
        if os.path.exists(self.data_file):

            print('READ FILE')
            # change the file name dat with asc
            file_split = self.data_file.split('.')
            file_name = file_split[0] + '.csv'

            # for data exported into a single csv-file
            if os.path.exists(file_name):
                print('check csv-file')
                file_ = open(file_name, 'r')
                header_line_1 = file_.readline().split()

                if header_line_1[0].split(';')[0] == 'Datum/Uhrzeit':
                    print('read csv-file')
                    # for data exported into down sampled data array
                    try:
                        _data_array = np.loadtxt(file_name,
                                                 delimiter=';',
                                                 skiprows=2)
                        # reset time[sec] in order to start at 0.
                        _data_array[:0] -= _data_array[0:0]
                    except ValueError:
                        # for first column use converter method 'time2sec';
                        converters = {0: time2sec}
                        # for all other columns use converter method
                        # 'comma2dot'
                        for i in range(len(header_line_1[0].split(';')) - 1):
                            converters[i + 1] = comma2dot
                        _data_array = np.loadtxt(
                            file_name, delimiter=";", skiprows=2, converters=converters)

                        # reset time[sec] in order to start at 0.
                        _data_array[:0] -= _data_array[0:0]

                else:
                    # for data exported into DAT and ASC-files
                    # try to use loadtxt to read data file
                    try:
                        _data_array = np.loadtxt(file_name,
                                                 delimiter=';')

                    # loadtxt returns an error if the data file contains
                    # 'NOVALUE' entries. In this case use the special
                    # method 'loadtxt_novalue'
                    except ValueError:
                        _data_array = loadtxt_novalue(file_name)

            if not os.path.exists(file_name):
                file_name = file_split[0] + '.ASC'
                if not os.path.exists(file_name):
                    raise IOError('file %s does not exist' % file_name)

                # for data exported into DAT and ASC-files
                # try to use loadtxt to read data file
                try:
                    _data_array = np.loadtxt(file_name,
                                             delimiter=';')

                # loadtxt returns an error if the data file contains
                # 'NOVALUE' entries. In this case use the special
                # method 'loadtxt_novalue'
                except ValueError:
                    _data_array = loadtxt_novalue(file_name)

            self.data_array = _data_array

    data_dir = Property()
    '''Local directory path of the data file.
    '''

    def _get_data_dir(self):
        return os.path.dirname(self.data_file)

    relative_path = Property
    '''Relative path inside database structure - the path is same for experiment
    in both database structures (remote and local)
    '''

    def _get_relative_path(self):
        return self.data_dir.replace(simdb.simdb_dir, '')[1:]

    hook_up_file = Property
    '''File specifying the access to extended data.
    The cfg file is used to hook up arbitrary type
    of data stored anywhere that can be downloaded
    on demand to the local cache.
    '''

    def _get_hook_up_file(self):
        dir_path = os.path.dirname(self.data_file)
        file_name = os.path.basename(self.data_file)
        file_split = file_name.split('.')
        file_name = os.path.join(dir_path,
                                 file_split[0] + '.cfg')
        if not os.path.exists(file_name):
            file_name = ''
        return file_name

    aramis_start_offset = Property(Float, depends_on='data_file')
    '''Get time offset of aramis start specified in the hookup file.
    '''
    @cached_property
    def _get_aramis_start_offset(self):
        # hook_up an extended file if available.
        aramis_start_offset = 0.0
        if self.hook_up_file:
            config = configparser.ConfigParser()
            config.read(self.hook_up_file)
            try:
                aramis_start_offset = config.get('aramis_data',
                                                 'aramis_start_offset')
            except configparser.NoOptionError:
                pass
        return float(aramis_start_offset)

    aramis_files = Property(depends_on='data_file')
    '''Get the list of available aramis files specified in the hookup file.
    '''
    @cached_property
    def _get_aramis_files(self):
        # hook_up an extended file if available.
        aramis_files = []
        if self.hook_up_file:
            config = configparser.ConfigParser()
            config.read(self.hook_up_file)
            aramis_files = config.get(
                'aramis_data', 'aramis_files').split(',\n')
        return aramis_files

    aramis_dict = Property(depends_on='data_file')
    '''Use the last two specifiers of the aramis file name
    as a key to access the proper file.
    '''
    @cached_property
    def _get_aramis_dict(self):
        # hook_up an extended file if available.
        af_dict = {}
        for af in self.aramis_files:
            fx, fy = af.split('-')[-2:]
            af_dict[fx + '-' + fy] = af
        return af_dict

    def download_aramis_file(self, arkey):
        af = self.aramis_dict[arkey]
        af_rel_dir = os.path.join(self.relative_path, 'aramis')
        af_local_dir = os.path.join(simdb.simdb_cache_dir, af_rel_dir)
        if not os.path.exists(af_local_dir):
            os.makedirs(af_local_dir)
        try:
            s = SFTPServer(simdb.server_username, '', simdb.server_host)
            if hasattr(s, 'sftp'):
                zip_filename = af + '.zip'
                zipfile_server = os.path.join(
                    simdb.simdb_cache_remote_dir, af_rel_dir, zip_filename)

                zipfile_server = string.replace(zipfile_server, '\\', '/')
                zipfile_local = os.path.join(af_local_dir, zip_filename)

                print('downloading', zipfile_server)
                print('destination', zipfile_local)

                s.download(zipfile_server, zipfile_local)
                s.sftp.stat(zipfile_server)
                s.close()
        except IOError as e:
            raise IOError(e)

    def uncompress_aramis_file(self, arkey):
        af = self.aramis_dict[arkey]
        af_rel_dir = os.path.join(self.relative_path, 'aramis')
        af_local_dir = os.path.join(simdb.simdb_cache_dir, af_rel_dir)
        zip_filename = af + '.zip'
        zipfile_local = os.path.join(af_local_dir, zip_filename)
        if not os.path.exists(zipfile_local):
            self.download_aramis_file(arkey)

        print('uncompressing')
        zf = zipfile.ZipFile(zipfile_local, 'r')
        zf.extractall(af_local_dir)
        zf.close()

    def get_cached_aramis_file(self, arkey):
        '''For the specified aramis resolution key check if the file
        has already been downloaded.
        '''
        af = self.aramis_dict.get(arkey, None)
        if af is None:
            print('Aramis data not available for resolution %s of the'\
                'test data\n%s' % (arkey, self.data_file))
            return None
        af_path = os.path.join(
            simdb.simdb_cache_dir, self.relative_path, 'aramis', af)

        if not os.path.exists(af_path):
            self.uncompress_aramis_file(arkey)

        return af_path



'''
Created on 14. 4. 2014

@author: Vancikv
'''

from traits.api import \
    Property, Button, Str, Event, WeakRef

from .matresdev.db.simdb import \
    SimDBClassExt

from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode

from traitsui.api import \
    View, Item, EnumEditor, Group

from bmcs_beam.mxn.mxn_class_extension_handler import \
    MxNClassExtHandler

class MxNClassExt(SimDBClassExt, MxNTreeNode):
    node_name = 'Material database'

    def __setitem__(self, key, value):
        value.key = key
        value.node_name = key
        super(MxNClassExt, self).__setitem__(key, value)

    new_material = Button(label='Add new material')
    def _new_material_fired(self):
        pass

    del_material = Button(label='Delete material')
    def _del_material_fired(self):
        pass

    save_database = Button(label='Save database')
    def _save_database_fired(self):
        for inst in list(self.instances.values()):
            inst.save()

    tree_node_list = Property(depends_on='instances')
    def _get_tree_node_list(self):
        for inst in self.inst_list:
            inst.node_name = inst.key
        return self.inst_list

    instance_keys = Property()
    def _get_instance_keys(self):
        return list(self.instances.keys())

    chosen_instance = Str()
    def _chosen_instance_default(self):
        return list(self.instances.keys())[0]

    tree_view = View(Item('new_material', show_label=False),
                     Item('save_database', show_label=False),
                     Group(
                     Item('chosen_instance', editor=EnumEditor(name='instance_keys'),
                          show_label=False),
                     Item('del_material', show_label=False)
                           )
                     , handler=MxNClassExtHandler())




'''
Created on 25. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property

from traitsui.api import \
    View, Item, Group

from .matrix_cross_section_geo import MCSGeo

import numpy as np


class MCSGeoI(MCSGeo):

    '''I-shaped cross section
    '''

    height = Float(0.6, auto_set=False, enter_set=True, geo_input=True)
    '''total height of crosssection
    '''

    height_up = Float(0.1, auto_set=False, enter_set=True, geo_input=True)
    '''height of upper flange
    '''

    height_lo = Float(0.1, auto_set=False, enter_set=True, geo_input=True)
    '''height of lower flange
    '''

    width_up = Float(0.4, auto_set=False, enter_set=True, geo_input=True)
    '''width of upper flange
    '''

    width_lo = Float(0.6, auto_set=False, enter_set=True, geo_input=True)
    '''width of lower flange
    '''

    width_st = Float(0.2, auto_set=False, enter_set=True, geo_input=True)
    '''width of stalk
    '''

    width = Property(depends_on='+geo_input')
    '''Width of cross section
    '''
    @cached_property
    def _get_width(self):
        return max(self.width_lo, self.width_up)

    gravity_centre = Property(depends_on='+geo_input')
    '''z distance of gravity centre from upper rim
    '''
    @cached_property
    def _get_gravity_centre(self):
        A_up, z_up = self.width_up * self.height_up, self.height_up / 2
        A_lo, z_lo = self.width_lo * \
            self.height_lo, self.height - self.height_lo / 2
        A_st, z_st = self.width_st * \
            (self.height - self.height_up - self.height_lo), (self.height +
                                                              self.height_up - self.height_lo) / 2
        return (A_up * z_up + A_lo * z_lo + A_st * z_st) / (A_up + A_lo + A_st)

    def get_width(self, z):
        '''Returns width of cross section for different vertical coordinates
        '''
        width = self.width_lo + (np.sign(z - self.height_lo) + 1) / 2 * (self.width_st - self.width_lo) + \
            (np.sign(z - self.height + self.height_up) + 1) / \
            2 * (self.width_up - self.width_st)
        return width

    width_vct = Property()

    @cached_property
    def _get_width_vct(self):
        return np.vectorize(self.get_width, otypes=[np.float])

    def plot_geometry(self, ax):
        '''Plot geometry'''
        w_max = self.width

        dx, dz = 0.0, 0.0

        xdata = np.array([-self.width_lo / 2, self.width_lo / 2, self.width_lo / 2, self.width_st / 2,
                          self.width_st / 2, self.width_up /
                          2, self.width_up / 2, -self.width_up / 2,
                          - self.width_up / 2, -self.width_st /
                          2, -self.width_st / 2,
                          - self.width_lo / 2, -self.width_lo / 2], dtype=float) + w_max / 2

        zdata = np.array([0, 0, self.height_lo, self.height_lo, self.height - self.height_up,
                          self.height - self.height_up, self.height, self.height, self.height -
                          self.height_up,
                          self.height - self.height_up, self.height_lo, self.height_lo, 0], dtype=float)

        ax.plot(xdata + dx, zdata + dz, color='blue')

        ax.axis('equal')
        ax.axis([dx - 0.1 * w_max, dx + 1.1 * w_max,
                 dz - 0.1 * self.height, dz + 1.1 * self.height])

    view = View(Item('width_up'),
                Item('width_lo'),
                Item('width_st'),
                Item('height'),
                Item('height_lo'),
                Item('height_up'),
                resizable=True,
                buttons=['OK', 'Cancel'])



from bmcs_beam.beam_config.beam_design import BeamDesign
from bmcs_beam.beam_config.boundary_conditions import BoundaryConditions, BoundaryConfig
from bmcs_beam.bending.deflection_profile import DeflectionProfile, LoadDeflectionParamsStudy


'''
Created on 25. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property

from traitsui.api import \
    View, Item, Group, HGroup

from .matrix_cross_section_geo import MCSGeo

import numpy as np


class MCSGeoRect(MCSGeo):

    '''Rectangular cross section
    '''
    height = Float(0.3, auto_set=False, enter_set=True, geo_input=True)
    '''total height of cross section
    '''

    width = Float(0.2, auto_set=False, enter_set=True, geo_input=True)
    '''total width of cross section
    '''

    gravity_centre = Property(depends_on='+geo_input')
    '''z distance of gravity centre from upper rim
    '''
    @cached_property
    def _get_gravity_centre(self):
        return self.height / 2

    def get_width(self, z):
        '''returns width of cross section for different vertical coordinates
        '''
        return self.width

    width_vct = Property()

    @cached_property
    def _get_width_vct(self):
        return np.vectorize(self.get_width, otypes=[np.float])

    def plot_geometry(self, ax):
        '''Plot geometry'''
        dx, dz = 0.0, 0.0
        xdata = np.array([0, self.width, self.width, 0, 0], dtype=float)
        zdata = np.array([0, 0, self.height, self.height, 0], dtype=float)
        ax.plot(xdata + dx, zdata + dz, color='blue')
        ax.axis('equal')
        ax.axis([dx - 0.1 * self.width, dx + 1.1 * self.width,
                 dz - 0.1 * self.height, dz + 1.1 * self.height])

    view = View(Item('width'),
                Item('height'),
                resizable=True,
                buttons=['OK', 'Cancel'])

if __name__ == '__main__':
    ge = MCSGeoRect()
    ge.configure_traits()



from .matrix_law_base import \
    MatrixLawBase

from .matrix_law_linear import \
    MatrixLawLinear

from .matrix_law_bilinear import \
    MatrixLawBilinear

from .matrix_law_block import \
    MatrixLawBlock

from .matrix_law_quadratic import \
    MatrixLawQuadratic

from .matrix_law_quad import \
    MatrixLawQuad



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Aug 7, 2009 by: rchx
'''
A trait type with shadow attribute that allows database referencing -
database keys are allowed as values, the shadow attribute is then the
appropriate database member.
'''

from traits.api import \
    TraitType, HasStrictTraits, TraitError, \
    Property, Button

from traitsui.api import \
     View, Item, EnumEditor

class KeyRef(TraitType):

    is_mapped = True

    def __init__(self, *args, **metadata):
        if len(args) == 1:
            self._default_key = args[0]
        else:
            self._default_key = None
        db = metadata.get('db', None)
        if db:
            self.map = db
        else:
            raise ValueError('db not defined')

        super(KeyRef, self).__init__(**metadata)

    def validate(self, obj, name, key):
        ''' Set the trait value '''

        self.keys_name = name + '_keys'
        keys_prop = Property(fget=lambda: list(self.map.keys()))
        obj.add_trait(self.keys_name, keys_prop)

        if key in list(self.map.keys()):
            prechange_mapped_val = getattr(obj, name + '_')
            try:
                prechange_mapped_val.del_link(obj)
                '''Management of backward links - erase the link to an object that has
                stopped using me.
                '''
            except AttributeError:
                print(('Warning: Unable to remove reference to object from '
                'mapped value of type %s (Backwards link management not implemented'
                ' in mapped type), trait name %s') % (prechange_mapped_val.__class__, name))
            postchange_mapped_val = self.mapped_value(key)
            try:
                postchange_mapped_val.add_link(obj)
                '''Management of backward links - add the link to an object that has
                started using me.
                '''
            except AttributeError:
                print(('Warning: Unable to pass reference to object to mapped '
                'value of type %s (Backwards link management not implemented '
                'in mapped type)') % prechange_mapped_val.__class__)

            self.post_setattr(obj, name, key)
            ''' When the new value is not different from the old,
            post_setattr does not get called, which can cause issues
            '''
            return key
        else:
            self.error(object, name, key)

    def get_default_value(self):
        '''Take the default value'''

        keys = list(self.map.keys())
        if self._default_key == None:
            if len(keys) > 1:
                return (0, list(self.map.keys())[0])
            else:
                return (0, None)
                raise TraitError('invalid key, no entries in database extension %s' % \
                    (self.map.klass,))
        if self._default_key in keys:
            return (0, self._default_key)

        raise TraitError('assigned default value must be one of %s but a value of %s was received' % \
            (list(self.map.keys()), self._default_key))

    def mapped_value (self, key):
        try:
            return self.map[ key ]
        except:
            return None

    def post_setattr (self, object, name, value):
        val = self.mapped_value(value)
        try:
            setattr(object, name + '_', val)
        except:
            # We don't need a fancy error message, because this exception
            # should always be caught by a TraitCompound handler:
            raise TraitError('Unmappable')

    def info (self):
        keys = [ repr(x) for x in list(self.map.keys()) ]
        keys.sort()
        return ' or '.join(keys)

    def get_editor (self, trait=None):
        return self.create_editor()

    def create_editor(self):
        return EnumEditor(name=self.keys_name)

if __name__ == '__main__':

    from bmcs_beam.mxn.reinf_laws import \
         ReinfLawBase, ReinfLawFBM

    class UseKeyRef(HasStrictTraits):
        '''Testclass containing attribute of type KeyRef
        '''
        new_law = Button
        def _new_law_fired(self):
            '''Adds a new fbm law to database and sets it as
            value of the ref attribute
            '''
            law_name = 'new_fbm_law_1'
            count = 1
            while ReinfLawBase.db.get(law_name, None):
                count += 1
                law_name = 'new_fbm_law_' + str(count)
            ReinfLawBase.db[law_name] = ReinfLawFBM()
            self.ref = law_name

            v = self.trait_view()
            v.updated = True
            '''View gets updated to acknowledge the change in database
            '''

        del_law = Button
        def _del_law_fired(self):
            '''Deletes currently selected law from database
            '''
            law_name = self.ref
            self.ref = 'fbm-default'
            ReinfLawBase.db.__delitem__(key=law_name)

            v = self.trait_view()
            v.updated = True
            '''View gets updated to acknowledge the change in database
            '''

        reinf_law_keys = Property
        def _get_reinf_law_keys(self):
            return list(ReinfLawBase.db.keys())

        ref = KeyRef(db=ReinfLawBase.db)

        traits_view = View(Item('ref'),
                           Item('new_law'),
                           Item('del_law'),
                          resizable=True)

    ukr = UseKeyRef()

    print('value of key')
    print(ukr.ref)
    print('value of ref')
    print(ukr.ref_)

#    print 'KE1', ukr.ref_keys

    ukr.ref = 'steel-default'

    ukr.add_trait('one', 1)
    print(ukr.one)

    print('KEYS', ukr.ref_keys)

    print('value of key')
    print(ukr.ref)
    print('value of ref')
    print(ukr.ref_)
#
    ukr.configure_traits()
#     '''View the testclass - the default database keys
#     are available in the dropdown list.
#     '''

    ReinfLawBase.db['FBM_keyref_test'] = ReinfLawFBM()
    '''Adding a new item to database.
    '''

    ukr.ref = 'FBM_keyref_test'
    '''Attribute of the testclass set to the new value.
    '''

    print('value of key')
    print(ukr.ref)
    print('value of ref')
    print(ukr.ref_)

#     ukr.configure_traits()
#     '''View the testclass - dropdown list should include
#     the new database item.
#     '''
#
#     ukr.configure_traits()
#     '''View the testclass - dropdown list should include
#     the new database item.
#     '''

    del ReinfLawBase.db['FBM_keyref_test']
#     ukr.ref = 'fbm_keyref_test'
#     '''New item got deleted - further attempt at
#     referencing it raises an error.
#     '''



'''
Created on 23. 4. 2014

@author: Vancikv
'''

from bmcs_beam.mxn.mxn_class_extension import \
    MxNClassExt
from bmcs_beam.mxn.reinf_laws import \
    ReinfLawBase, ReinfLawBilinear, \
    ReinfLawCubic, ReinfLawFBM, \
    ReinfLawLinear
from traits.api import \
    Property, cached_property, Dict, Str, \
    Float, Trait
from traitsui.api import \
    View, Item, EnumEditor

from .material_type_base import \
    MaterialTypeBase
from .material_type_handler import \
    MaterialTypeHandler


class MTReinfFabric(MaterialTypeBase):

    A_roving = Float(0.461, auto_set=False, enter_set=True, geo_input=True)
    '''Cross section of one roving [mm**2]'''

    s_0 = Float(0.0083, auto_set=False, enter_set=True, geo_input=True)
    '''Distance between rovings oriented in 0-direction [m]'''

    s_90 = Float(0.02, auto_set=False, enter_set=True, geo_input=True)
    '''Distance between rovings oriented in 90-direction [m]'''

    mtrl_laws = Dict((Str, ReinfLawBase))

    def _mtrl_laws_default(self):
        basic_laws = {'bilinear':
                      ReinfLawBilinear(sig_tex_u=1216., eps_u=0.014,
                                       var_a=0.8, eps_el_fraction=0.0001),
                      'cubic':
                      ReinfLawCubic(sig_tex_u=1216., eps_u=0.016,
                                    var_a=-5e+6),
                      'fbm':
                      ReinfLawFBM(sig_tex_u=1216., eps_u=0.014,
                                  m=0.5),
                      'linear':
                      ReinfLawLinear(eps_u=0.014, E_tex=80000.),
                      }

        return basic_laws

    possible_laws = {'fbm': ReinfLawFBM, 'linear': ReinfLawLinear,
                     'cubic': ReinfLawCubic, 'bilinear': ReinfLawBilinear}

    named_mtrl_laws = Property(depends_on='mtrl_laws')

    @cached_property
    def _get_named_mtrl_laws(self):
        for key, mtrl_law in list(self.mtrl_laws.items()):
            mtrl_law.node_name = key
        return self.mtrl_laws

    #=========================================================================
    # UI-related functionality
    #=========================================================================

    tree_view = View(Item('A_roving'),
                     Item('s_0'),
                     Item('s_90'),
                     Item('new_law', show_label=False),
                     Item('chosen_law',
                          editor=EnumEditor(name='law_keys'),
                          show_label=False),
                     Item('del_law', show_label=False),
                     handler=MaterialTypeHandler()
                     )

MTReinfFabric.db = MxNClassExt(
    klass=MTReinfFabric,
    verbose='io',
    node_name='Reinforcement fabrics'
)

if MTReinfFabric.db.get('default_fabric', None):
    del MTReinfFabric.db['default_fabric']
MTReinfFabric.db['default_fabric'] = MTReinfFabric()





from traits.etsconfig.api import ETSConfig

if ETSConfig.toolkit == 'wx':
    from .mpl_figure_editor_wx import MPLFigureEditor
if ETSConfig.toolkit == 'qt4':
    from .mpl_figure_editor_qt import MPLFigureEditor
else:
    raise ImportError("MPLEditor for %s toolkit not availabe" % \
        ETSConfig.toolkit)



#-------------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Dec 21, 2011 by: rch

from traits.api import \
    HasTraits, Str, Property, cached_property, \
    Enum

import os.path

from os.path import expanduser

HOME_DIR = expanduser("~")
# build directory
BUILD_DIR = os.path.join(HOME_DIR, '.mxn', 'docs')
# output directory for the documentation
DOCS_DIR = os.path.join('..', 'docs', )
# output directory for the example documentation
EX_OUTPUT_DIR = os.path.join(DOCS_DIR, '_component_modules')

class GenExampleDoc(HasTraits):

    header = Str('''
%s
================================

    ''')

    component_module = None

    #===========================================================================
    # Derived traits
    #===========================================================================
    component_obj = Property(depends_on='component_module')
    @cached_property
    def _get_component_obj(self):
        return self.component_module.create_doc_object()

    name = Property(depends_on='component_module')
    @cached_property
    def _get_name(self):
        return self.component_obj.__class__

    output_dir = Property(depends_on='component_module')
    @cached_property
    def _get_output_dir(self):
        return os.path.join(EX_OUTPUT_DIR, self.name)

    rst_file_name = Property(depends_on='component_module')
    @cached_property
    def _get_rst_file_name(self):
        return os.path.join(self.output_dir, 'index.rst')

    def generate_html(self):

        print('generating documentation for', self.name, '...')

        rst_text = '''
================================
Parametric study for %s
================================
        ''' % self.name

        dobj = self.component_obj

        if dobj.s.q.__doc__ != None:
            rst_text += dobj.s.q.__doc__

        rst_text += self.header

        for st in dobj.sampling_types:
            rst_text += '''

.. image:: %s_%s.png
    :width: 24%%

            ''' % (self.name, st)

        for st in dobj.sampling_types:
            rst_text += '''

.. image:: %s_sampling_%s.png
    :width: 24%%

            ''' % (self.name, st)

        rst_text += '\nFollowing oricrete configuration has been used to produce the sampling figures:\n\n'
        rst_text += '\n>>> print component_obj\n' + str(dobj.s) + '\n'

        rst_text += '''
Comparison of execution time for different sampling types
=========================================================
Execution time evaluated for an increasing number of sampling points n_sim:
'''
        for basename in dobj.fnames_sampling_efficiency:
            rst_text += '''

.. image:: %s
    :width: 100%%

            ''' % basename
            print('written file %s', basename)

        rst_text += '\n'

        rst_text += '''
Comparison of efficiency for different code types
=========================================================
Execution time evaluated for an numpy, weave and cython code:
'''
        for basename in dobj.fnames_language_efficiency:
            rst_text += '''

.. image:: %s
    :width: 100%%

            ''' % basename
            print('written file %s', basename)

        rst_text += '\n'

        rst_file = open(self.rst_file_name, 'w')

        rst_file.write(rst_text)

        rst_file.close()

class GenDoc(HasTraits):
    '''
    Configuration of the document generation using sphinx.
    '''
    component_modules = []

    build_mode = Enum('local', 'global')

    build_dir = Property(depends_on='build_mode')
    def _get_build_dir(self):
        build_dir = {'local' : '.',
                     'global' : BUILD_DIR }
        return build_dir[self.build_mode]

    html_server = 'root@mordred.imb.rwth-aachen.de:/var/www/docs/oricrete'

    method_dispatcher = {'all' : 'generate_examples' }

    def generate_html(self):
        for demo in self.component_modules:
            ged = GenExampleDoc(component_module=demo)
            ged.generate_html()

        os.chdir(DOCS_DIR)
        sphings_cmd = 'sphinx-build -b html -E . %s' % self.build_dir
        os.system(sphings_cmd)

    def push_html(self):
        '''
        Push the documentation to the server.
        '''
        rsync_cmd = 'rsync -av --delete %s/ %s' % (self.build_dir, self.html_server)
        os.system(rsync_cmd)

if __name__ == '__main__':

    gd = GenDoc(build_mode='global')
    gd.generate_html()
    # gd.push_html()



import matplotlib.patches as mpatches
import numpy as np
import sympy as sp
import traits.api as tr
from bmcs_cross_section.mkappa import MKappa
from bmcs_beam.beam_config.boundary_conditions import BoundaryConditions
from bmcs_utils.api import InteractiveModel, \
    Item, View, Float, Int, FloatEditor, FloatRangeEditor, mpl_align_yaxis
from matplotlib.patches import PathPatch
from matplotlib.path import Path
from sympy.physics.continuum_mechanics.beam import Beam

class UltimateLimitState(InteractiveModel):

    F_max = Float(1, desc='maximum load value', BC=True)
    theta_F = Float(1.0, desc='load factor', BC=True)

    F = tr.Property(depends_on='+BC')

    ipw_view = View(
        Item('theta_F', latex=r'\theta [-]',
             editor=FloatRangeEditor(low=0, high=1)),
        Item('F_max', latex='F_\mathrm{max} [\mathrm{N}]',
             readonly=True),
        Item('G_adj', param=True, latex='G_{adj} [\mathrm{-}]', minmax=(1e-3, 1e-1)),
        Item('n_x', param=True, latex='n_x [\mathrm{-}]'),
    )

    x = tr.Property(depends_on='_GEO')

    @tr.cached_property
    def _get_x(self):
        pass

    def get_M_x(self):
        pass

    def plot_geo(self, ax1):
        pass

    def subplots(self, fig):
        pass

    def update_plot(self, axes):
        ax1, ax2, ax3 = axes
        self.plot_geo(ax1)
        self.plot_MQ(ax2, ax3)
        ax1.axis('equal')
        ax1.autoscale(tight=True)

class ServiceabilityLimitState(InteractiveModel):

    F_max = Float(1, desc='maximum load value', BC=True)
    theta_F = Float(1.0, desc='load factor', BC=True)

    F = tr.Property(depends_on='+BC')

    ipw_view = View(
        Item('theta_F', latex=r'\theta [-]',
             editor=FloatRangeEditor(low=0, high=1)),
        Item('F_max', latex='F_\mathrm{max} [\mathrm{N}]',
             readonly=True),
        Item('G_adj', param=True, latex='G_{adj} [\mathrm{-}]', minmax=(1e-3, 1e-1)),
        Item('n_x', param=True, latex='n_x [\mathrm{-}]'),
    )

    x = tr.Property(depends_on='_GEO')

    @tr.cached_property
    def _get_x(self):
        pass

    def get_M_x(self):
        pass

    def plot_geo(self, ax1):
        pass

    def subplots(self, fig):
        ax1, ax2 = fig.subplots(2, 1)
        ax3 = ax2.twinx()
        return ax1, ax2, ax3

    def update_plot(self, axes):
        ax1, ax2, ax3 = axes
        self.plot_geo(ax1)
        self.plot_MQ(ax2, ax3)
        ax1.axis('equal')
        ax1.autoscale(tight=True)





#From MATLAB code


# % ANALYSIS AND DESIGN OF RECTANGULAR AND FLANGED BEAMS PER EUROCODE 2
# % THIS PROGRAM WILL CALCULATE THE AREA OF REINFORCEMENT REQUIRED, AND ALSO DO THE DEFLECTION VERIFICATION USING THE DEEMED TO SATISFY RULES OF EC2
#
#
# % INPUT MATERIAL PROPERTIES
Fck = 1 #(‘Enter the grade of concrete (N/mm^2)Fck  = ‘);
Fyk = 1 #(‘Enter the yield strength of steel (N/mm^2)Fyk = ‘);
#
# % INPUT DESIGN MOMENT
MEd = 1 #(‘Enter the ultimate design moment (KNm) MEd = ‘);

# % INPUT SECTION AND DESIGN PROPERTIES
h = 1 #(‘Enter the depth of beam (mm)h = ‘);
b = 1 #(‘Enter the effective flange width of the beam (mm)b = ‘);
bw = 1 #(‘Enter the beam width (mm)bw = ‘);
Cc = 1 #(‘Enter concrete cover (mm) = ‘);
dr = 1 #(‘Enter the diameter of reinforcement (mm) = ‘);
dl = 1 #(‘Enter the diameter of links = ‘);

# % CALCULATION OF EFFECTIVE DEPTH

d = h-Cc-(dr/2)-dl # ‘Effective depth d (mm)’

do = Cc+dl+(dr/2) # ‘Depth of reinforcement from the face of concrete’

# % ANALYSIS FOR INTERNAL STRESSES
ko = 0.167
k = (MEd*10^6)/(Fck*b*d**2)
if k>ko:

    # disp(‘Since k > ko, Compression reinforcement is required’)
    Mcd = (Fck*b*d**2*(k-ko))*10**(-6)
    As2 = (Mcd*10**6)/(0.87*Fyk*(d-do))
    z = 0.5*d*(1+(1-3.53*ko) ** 0.5)
    As1 = ((ko*Fck*b*d^2)/(0.87*Fyk*z))+ As2
else:
    # disp(‘Since k < ko, No Compression reinforcement is required’)
    # disp(‘Lever arm (la)’)
    la = 0.5+ (0.25-0.882*k) ** 0.5
    if la>0.95:
        # disp(‘Since la > 0.95,’)
        la = 0.95
    else:
        la = 0.5+ (0.25-0.882*k)**0.5
        As1 =(MEd*10^6)/(0.87*Fyk*la*d)


# % MINIMUM AREA OF STEEL REQUIRED
fctm = 0.3*(Fck **(2/3)) #%MEAN TENSILE STRENGTH OF CONCRETE (TABLE 3.1 EC2)
ASmin = 0.26*(fctm/Fyk)*bw*d
if ASmin < 0.0013*bw*d:
    ASmin = 0.0013*bw*d
elif As1<ASmin:
    As1 = ASmin
else:
    print('Since As1 > Asmin, provide As1 which is the area of steel required')



Asprov1 = 1 #(‘Enter area of tension steel provided (mm^2) = ‘);
Asprov2 = 1 #(‘Enter area of compression steel provided if any (mm^2) = ‘);



# % CHECK FOR DEFLECTION
#
# disp(‘DO YOU WANT TO CHECK FOR DEFLECTION?’)
# G = input(‘ENTER (1) FOR YES OR (0) FOR NO = ‘);
# if G == 1;
#     disp(‘CHECK FOR DEFLECTION’)
# disp(‘BASIC SPAN/EFFECTIVE DEPTH RATIO (K)’)
# disp(‘CANTILEVER = 0.4’)
# disp(‘SIMPLY SUPPORTED = 1.0’)
# disp(‘SIMPLY SUPPORTED AND FIXED AT ONE END = 1.3’)
# disp(‘FIXED AT BOTH ENDS = 1.5’)
# K = input(‘Enter the selected value of k = ‘);
# L = input(‘Enter the deflection critical length of the member (mm) = ‘);
#
# disp(‘Fs is the stress in tensile reinforcement under service loading’)
# Fs = (310*Fyk*As1)/(500*Asprov1)
# disp(‘Bs = 310/Fs’)
# Bs = 310/Fs
# disp(‘P is the reinforcement ratio in the section (Asprov1)/(b*d)’)
# P = (Asprov1)/(b*d)
# disp(‘Po = sqrt(Fck)/1000’)
# Po = sqrt(Fck)/1000
# P1 = (Asprov2)/(b*d)
# if
#   P <= Po
#   I_deflection = K*(11+(1.5*sqrt(Fck)*(Po/P)) + 3.2*sqrt(Fck)*((Po/P)-1)^1.5)
# else
#     I_deflection = K*(11+(1.5*sqrt(Fck)*(Po/(P-P1))) + 3.2*sqrt(Fck)*((Po/P)-1)^1.5)
# end
# CV = input(‘Is the beam is flanged? (1) for YES and (0) for NO = ‘)
# if CV == 1
# disp(‘S is the ratio of beff/bw’)
# S = b/bw
#   bn = (11-(b/bw))/10
#
#   if S >= 3.0;
#    K_deflection = Bs*0.8* I_deflection
# else
#    K_deflection = Bs*bn* I_deflection
#   end
#    if L>7000
#     Limiting_deflection = (7000/L)*K_deflection
# else
#     Limiting_deflection =  K_deflection
# end
#   end
# else
#   Limiting_deflection =   Bs*I_deflection
# end
# Actual_deflection = L/d
# if
# Limiting_deflection > Actual_deflection
#     disp(‘Deflection is satisfactory’)
# else
#     disp(‘DEFLECTION IS NOT SATISFACTORY !!!!! Increase depth of beam, or increase area of steel, or both. Then rerun proforma’)
#
#
# end
#
# 2.0 MATLAB CODE FOR SHEAR DESIGN ACCORDING TO EC2
#
# % SHEAR DESIGN IN EUROCODE 2
# clc
# disp(‘SHEAR DESIGN ACCORDING TO EC2′)
# disp(‘THIS PROFORMA WAS WRITTEN BY O.U.R. UBANI’)
#
#
# % MATERIALS PROPERTIES
# Fck = input(‘Enter the grade of concrete (N/mm^2) = ‘);
# Fyk = input(‘Enter the yield strength of steel (N/mm^2) = ‘);
#
# VEd = input(‘Enter the value of shear force at ULS (KN) = ‘);
#
# % SECTION PROPERTIES
# h = input(‘Enter the depth of beam (mm) = ‘);
# bw = input(‘Enter the beam width (mm) = ‘);
# Cc = input(‘Enter concrete cover (mm) = ‘);
# dr = input(‘Enter the diameter of reinforcement (mm) = ‘);
# dl = input(‘Enter the diameter of links = ‘)
# d = h-Cc-(dr/2)-dl % Effective depth
# do = Cc+dl+(dr/2)
#
# % CALCULATION OF THE SHEAR CAPACITY OF THE SECTION WITH NO SHEAR REINFORCEMENT
#
# Asprov1= input(‘Enter the area of steel provided in the shear zone (mm^2) = ‘) % The reinforcement must exceed the design anchorage length by at least the effective depth
# Crd = 0.12
# k1= 1+sqrt(200/d)
# if k1>2
# k1=2
# end
# disp(‘Reinforcement ratio’)
# P1 = (Asprov1/(bw*d))
# if P1>0.02
# P1 = 0.02
# end
#
# % THIS SECTION IS TO BE CONSIDERED IF THERE IS AXIAL FORCE IN THE SECTION
# disp(‘Axial force in the section’)
# N = input(‘Enter the value of AXIAL FORCE IF ANY(+VE FOR COMP, AND -VE FOR TENSION)(kN) = ‘)
# disp(‘Axial stress in the section (Ds)’)
# Ds = (N*1000)/(bw*h)
# if Ds > (0.2*0.85*Fck)/1.5
# Ds = (0.2*0.85*Fck)/1.5
# end
# k2 = 0.15;
# disp(‘Minimum shear stress in the section (N/mm^2)’)
# Vmin = 0.035*k1^(1.5)*sqrt(Fck)
# disp(‘Concrete resistance shear stress (VRd) (N/mm^2)’)
# VRdQ = ((Crd*k1)*(100*P1*Fck)^(1/3)*(bw*d))/1000 % SHEAR FORCE CONTRIBUTION
# VRdN = ((k2*Ds)*(bw*d))/1000 % AXIAL FORCE CONTRIBUTION
# VRd = VRdQ + VRdN % TOTAL SHEAR RESISTANCE
# if VRd < ((Vmin+(k2*Ds))*(b*d))/1000
# VRd = ((Vmin+(k2*Ds))*(b*d))/1000
# end
# if VRd>VEd
# disp(‘Since VRd > VEd’)
# disp(‘NO SHEAR REINFORCEMENT REQUIRED, PROVIDE NOMINAL LINKS’)
# else
# if VRd<VEd
# disp(‘Since  VRd < VEd’)
# disp(‘SHEAR REINFORCEMENT REQUIRED, CALCULATE COMPRESSION STRUT CAPACITY’)
# disp(‘Assume strut angle (thetha) = 21.8 deg, cot(thetha)= 2.5’)
# thetha = 21.8
# V1 = 0.6*(1-(Fck/250))
# disp(‘Design compressive strength of concrete (fcd) (N/mm^2)’)
# fcd = 0.567*Fck
# z = 0.9*d
# disp(‘Maximum capacity of compression strut (KN)’)
# VRDmax = ((b*z*V1*fcd)/(2.9))/1000
# if VRDmax > VEd
# disp(‘Since VRDmax > VEd’)
# disp(‘OK! Calculate diameter and spacing of links’)
# ASMINlinks_to_spacing = ((0.08*sqrt(Fck))/Fyk)*b
# ASlinks_to_spacing = (VEd*1000)/(z*0.87*Fyk*2.5)
# else
# disp(‘Since VRDmax > VEd, it means we need a higher strut angle (beta)’)
# disp(‘By calculating the strut angle’)
# disp(‘Shear stress at the section (N/mm^2)’)
# v = (VEd*1000)/(b*d)
# beta = (0.5)*asind((v)/(0.153*Fck*(1-(Fck/250))))
# if beta>45
# disp(‘Since beta < 45 deg, SECTION INADEQUATE FOR SHEAR, INCREASE DEPTH !!!!!’)
#    else
# disp(‘The ratio of Area of steel/spacing of links’)
# ASlinks_to_spacing = (VEd*1000)/(z*0.87*Fyk*cotd(beta))
# end
# end
# end
# Area_of_legs = input(‘Enter the area of number of legs selected = ‘)
# Spacing = input(‘Enter the spacing = ‘)
# disp(‘CHECK’)
# T = Area_of_legs/Spacing
# if T > ASlinks_to_spacing
# disp(‘Shear reinforcement is ok’)
# else
# disp(‘Increase area of steel, or reduce spacing’)
# end
# end


#-------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license. The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Mar 29, 2010 by: rch

from os.path import \
    join
import os.path
import platform
import sys
from traits.api import \
    HasTraits, Property, Str, Constant
from traits.util.home_directory import \
    get_home_directory
if platform.system() == 'Linux':
    pathchar = '/'
elif platform.system() == 'Windows':
    pathchar = '\\'


class SimDB(HasTraits):
    '''
Basic structure of the database directory.
Implements the relative paths for the three different
categories of data that are managed using the svn server.
- experimental data
- simulation data
- material data
Repository of raw data
- local data
'''
    home_dir = Property

    def __init__(self, *args, **kw):
        super(SimDB, self).__init__(*args, **kw)
        sys.path.append(self.exdata_dir)

    def _get_home_dir(self):
        return get_home_directory()

    simdb_dir = Property

    def _get_simdb_dir(self):
        return join(self.home_dir, 'simdb')

    exdata_dir = Property

    def _get_exdata_dir(self):
        return join(self.simdb_dir, 'exdata')

    matdata_dir = Property

    def _get_matdata_dir(self):
        return join(self.simdb_dir, 'matdata')

    simdata_dir = Property

    def _get_simdata_dir(self):
        return join(self.simdb_dir, 'simdata')

    report_dir = Property
    '''Directory for generated reports.
    '''

    def _get_report_dir(self):
        simdb_reports = join(self.home_dir, 'simdb_reports')
        if not os.path.exists(simdb_reports):
            os.makedirs(simdb_reports)
        return simdb_reports
    '''
    -remote data
    '''
    server_username = Str('simdb')
    '''User name to access data on the remote data server.
    '''

    server_host = Str('mordred.imb.rwth-aachen.de')
    '''Host name of the data repository.
    '''

    simdb_cache_remote_dir = Str('/home/simdb/simdb/')
    '''Root directory on the remote server.
    '''

    simdb_cache_dir = Property
    '''Cache directory for downloaded data.
    '''

    def _get_simdb_cache_dir(self):
        return join(self.home_dir, '.simdb_cache')

    pathchar = Constant(pathchar)

simdb = SimDB()



'''
Created on Jan 21, 2014

@author: rch
'''

from traits.api import HasStrictTraits, \
    Event, on_trait_change, Instance, Button

class MCSGeo(HasStrictTraits):
    '''Base class for cross section types.
    '''
    changed = Event
    @on_trait_change('+geo_input')
    def set_changed(self):
        self.changed = True
      
    def plot_geometry(self, ax):
        '''Plot geometry'''





from .reinf_law_base import \
    ReinfLawBase

from .reinf_law_bilinear import \
    ReinfLawBilinear

from .reinf_law_cubic import \
    ReinfLawCubic

from .reinf_law_piecewise_linear import \
    ReinfLawPiecewiseLinear

from .reinf_law_fbm import \
    ReinfLawFBM

from .reinf_law_linear import \
    ReinfLawLinear

from .reinf_law_steel import \
    ReinfLawSteel




'''
Created on Jan 30, 2014

@author: rch

Example of cross section assembly - Rectangular concrete
cross section with steel reinforcement
'''

from bmcs_beam.mxn import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

from bmcs_beam.mxn.reinf_layout import \
    RLCBar

from matplotlib.figure import \
    Figure

from matplotlib.backends.backend_agg import \
    FigureCanvasAgg

from bmcs_beam.mxn.utils import \
    get_outfile

bar = RLCBar(x=0.1, z=0.05, material='bar_d10',
             material_law='steel')
'''Single steel reinforcement bar
'''

ge = MCSGeoRect(height=0.5, width=0.3)
'''Cross section geometry
'''

mcs = MatrixCrossSection(geo=ge,
                         n_cj=20,
                         material='default_mixture',
                         material_law='constant')
'''Matrix cross section
'''

cs = CrossSection(reinf=[bar],
                  matrix_cs=mcs,
                  eps_lo=0.002,
                  eps_up=-0.0033
                  )

print('normal force', cs.N)
print('moment', cs.M)

fig = Figure(figsize=(10, 7), dpi=80, facecolor='white')
canvas = FigureCanvasAgg(fig)
ax = fig.add_subplot(1, 1, 1)
cs.plot_geometry(ax)

canvas.print_figure(get_outfile(folder_name='.mxn',
                                file_name='ex01.png'))
'''Saves a picture of cross section to a directory .mxn
located in user's home directory
'''



'''
Created on 15. 2. 2014

@author: Vancikv
'''

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect, MCSGeoI

from bmcs_beam.mxn.reinf_layout import \
    RLCBar, RLCTexLayer

import numpy as np

def test_cross_section_mn():
    '''Test the moment and normal force calculated for a cross section.
    I - shaped cross section with mixed reinforcement. Change of geometry
    to rectangular also tested.
    '''
    ge = MCSGeoI(height=0.4, height_up=0.05, width_up=0.25, height_lo=0.05, width_lo=0.35, width_st=0.05)
    mcs = MatrixCrossSection(geo=ge, n_cj=20, material='default_mixture',
                             material_law='constant')
    '''Cross section geometry + matrix
    '''

    bar1 = RLCBar(x=0.025, z=0.025, material='bar_d10')
    bar2 = RLCBar(x=0.125, z=0.025, material='bar_d10')
    bar3 = RLCBar(x=0.225, z=0.025, material='bar_d10')
    bar4 = RLCBar(x=0.325, z=0.025, material='bar_d10')
    bar1.material_.area = 0.00005
    '''Four steel reinforcement bars in lower flange
    '''

    tl1 = RLCTexLayer(z_coord=0.01, material='default_fabric', material_law='fbm')
    tl2 = RLCTexLayer(z_coord=0.39, material='default_fabric', material_law='fbm')
    '''Two layers of textile reinforcement
    '''

    cs = CrossSection(reinf=[tl1, tl2, bar1, bar2, bar3, bar4],
                             matrix_cs=mcs,
                             eps_lo=0.002,
                             eps_up=-0.0033,
                             )

    tl1.material_law_.set(sig_tex_u=1216., eps_u=0.014, m=0.5)
    tl1.material_.set(s_0=0.02, A_roving=0.461)

    assert np.allclose([cs.M, cs.N], [201.35521782599423, -1152.7647363907902])
    bar1.material_.area = 0.0004
    assert np.allclose([cs.M, cs.N], [274.03855115932754, -685.51473639079018])
    ge.height_lo = 0.06
    assert np.allclose([cs.M, cs.N], [279.41793763239417, -685.51473639079018])
    cs.eps_lo = 0.004
    assert np.allclose([cs.M, cs.N], [324.31926663193633, -323.07965392270125])
    mcs.geo = MCSGeoRect(height=0.4, width=0.4)
    assert np.allclose([cs.M, cs.N], [613.20118671511852, -2927.1270652809549])

if __name__ == '__main__':
    test_cross_section_mn()



'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Property, cached_property

import numpy as np

from bmcs_beam.mxn.mfn import MFnLineArray

from .matrix_law_base import \
    MatrixLawBase


class MatrixLawBilinear(MatrixLawBase):

    '''Effective crack bridge Law based on fiber-bundle-model.'''
    #-----------------------------
    # for bilinear stress-strain-diagram of the concrete (EC2)
    #-----------------------------
    mfn = Property(depends_on='+input')

    @cached_property
    def _get_mfn(self):
        '''bilinear stress-strain-diagram of the concrete
        '''
        # (for standard concrete)
        eps_c3 = self.f_ck / self.E_c
        xdata = np.array([0.0, eps_c3, self.eps_c_u])
        ydata = np.array([0.0, self.f_ck, self.f_ck])
        return MFnLineArray(xdata=xdata, ydata=ydata, extrapolate='zero')

if __name__ == '__main__':
    MatrixLawBase.db.configure_traits()



# Import Standard Libraries
import logging
import scipy as np

# Import Local Libraries
from Utilities import *

#===========================================================================
#   ACI Equations - Material properties
#===========================================================================


def elastic_modulus(fck, rho=145, units="psi"):
    """ Input:  fck = char. comp. strength of concrete
                rho = density of concrete (default = 145)
                units = "MPa" or "psi" (default = "psi") 
        Output: Ec = mean elastic modulus of concrete """
    fck = convert_2_psi(fck, units)
    rho = convert_2_pcf(rho, units)
    Ec = 33*rho**1.5*np.sqrt(fck)
    return Ec if units == "psi" else convert_2_MPa(Ec, "psi")


def tensile_strength(fck, units="psi"):
    """ Input:  fck = char. comp. strength of concrete 
                units = "MPa" or "psi" (default = "psi")
        Output: fctm = mean tensile strength of concrete """
    fck = convert_2_psi(fck, units)
    la = 1  # lambda factor necessary
    fctm = 7.5*la*np.sqrt(fck)
    return fctm if units == "psi" else convert_2_MPa(fctm, "psi")


def ultimate_strain(fck, units="psi"):
    """ Input:  fck = char. comp. strength of concrete
                units = "MPa" or "psi" (default = "psi") 
        Output: ecu = ultimate strain of concrete """
    fck = convert_2_psi(fck, units)
    return 3/1000

#===========================================================================
#   ACI Equations - Parameters
#===========================================================================


def beta(fck, units="psi"):
    """ Input:  fck = char. comp. strength of concrete
                units = "MPa" or "psi" (default = "psi") 
        Output: beta_1 = a/c in Whitney stress block """
    fck = convert_2_psi(fck, units)
    beta1 = 0.85-0.05*(fck-4000)/1000
    return min(max(beta1, 0.65), 0.85)


#===========================================================================
#   ACI Equations - Maximum reinforcement (Ductility)
#===========================================================================

def ductility_requirement(c, d, type="beam"):
    """ Input:  e_T = avg. tensile strain of steel
                type = "beam", "ties" or "spiral"
        Output: phi = strength reduction factor """
    e_T = (0.003/c)*(d-c)
    if type == "beam" and e_T >= 0.004:
        phi = 0.65 + 0.25/0.003*(e_T - 0.002)
        logging.info("    e_T = {:6.5f} > e_T,min = 0.004. OK".format(e_T))
        return min(phi, 0.9)
    elif type == "ties":
        phi = 0.65 + 0.25/0.003*(e_T - 0.002)
        return min(max(phi, 0.65), 0.9)
    elif type == "spiral":
        phi = 0.75 + 0.15/0.003*(e_T - 0.002)
        return min(max(phi, 0.75), 0.9)
    else:
        logging.info("    e_T = {:6.5f} < e_T,min = 0.004. Not OK".format(e_T))

#===========================================================================
#   ACI Equations - Minimum reinforcement (Md > Mcr)
#===========================================================================

def steel_ratio(As, fck, fyk, b, d, units="psi"):
    """ Input:  As = area of reinforcement steel
                fck = char. comp. strength of concrete
                fyk = char. yield stress of reinforcement
                b = width of beam portion in compression
                d = distance from comp. to reinforcement 
                units = "MPa" or "psi" (default = "psi")
        Output: A_min = minimum reinforcment area
                A_max = maximum reinforcement area """
    [fck, fyk] = convert_2_psi(np.array([fck, fyk]), units)
    [b, d] = convert_2_in(np.array([b, d]), units)
    As = convert_2_in2(As, units)

    A_min = max(3*np.sqrt(fck)/fyk, 200/fyk) * (b*d)
    A_max = 0.365*(fck/fyk)*beta(fck) * (b*d)

    compare_steel_area(As, A_min, A_max)

    return [A_min, A_max] if units == "psi" else [A_min*25.4**2, A_max*25.4**2]

#===========================================================================
#   ACI Equations - Design Aids
#===========================================================================


def barsize(Nbar, units="psi"):
    """ bar_area = barsize(Nbar or bar_area) """
    bar_area = {3: 0.11, 4: 0.20, 5: 0.31, 6: 0.44, 7: 0.60,
                8: 0.79, 9: 1, 10: 1.27, 11: 1.56, 14: 2.25, 18: 4.00}
    return bar_area.get(Nbar, Nbar)


def min_beam_width(numbar, rebar, units="psi"):
    """ min_width = min_beam_width(numbar,rebar) """
    if numbar > 1 & numbar < 7 & units == "psi":
        BW = [[7.0, 8.5, 10.0, 11.5, 13.0],
              [7.0, 8.5, 10.5, 12.0, 13.5],
              [7.0, 9.0, 11.0, 12.5, 14.0],
              [7.5, 9.0, 11.0, 13.0, 15.0],
              [7.5, 9.5, 11.5, 13.5, 15.5],
              [8.0, 10.0, 12.5, 14.5, 17.0],
              [8.0, 10.5, 13.0, 15.5, 18.0],
              [8.5, 11.0, 14.0, 17.0, 19.5]]
        return BW[rebar-4, numbar-2]
    else:
        raise ValueError()





'''
Created on Sep 4, 2012

@author: rch
'''
from traits.api import \
    Int, Instance, Property, List, \
    cached_property, Event, on_trait_change

from traitsui.api import \
    View, Item, Group, HGroup, RangeEditor

from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode

from bmcs_beam.mxn.cross_section import \
    CrossSection

from bmcs_beam.mxn.reinf_layout import \
    RLCTexUniform

import numpy as np

from bmcs_beam.mxn.matrix_cross_section import \
    MatrixCrossSection, MCSGeoRect

class MxNDiagram(MxNTreeNode):

    modified = Event
    def set_modified(self):
        self.modified = True

    tree_node_list = List(Instance(CrossSection))
    def _tree_node_list_default(self):
        return [CrossSection(notify_change_ext=self.set_modified,
                            reinf=[RLCTexUniform(
                                      n_layers=12,
                                      material='default_fabric')],
                            matrix_cs=MatrixCrossSection(geo=MCSGeoRect(
                                      width=0.2, height=0.06), n_cj=20,
                                      material='default_mixture',
                                      material_law='constant'))]

    @on_trait_change('tree_node_list')
    def cs_changed(self):
        self.modified = True

    cs = Property(depends_on='tree_node_list')
    def _get_cs(self):
        val = self.tree_node_list[0]
        val.notify_change_ext = self.set_modified
        return val
    def _set_cs(self, val):
        self.tree_node_list = [val]

    eps_cu = Property()
    def _get_eps_cu(self):
        return -self.cs.matrix_cs_with_state.material_law_.eps_c_u

    strain_env_reinf = Property()
    '''Contains the reinforcement element that determines the strain envelope.
    '''
    def _get_strain_env_reinf(self):
        eps_lo_arr = np.array([[r, r.convert_eps_u_2_lo(eps_up=self.eps_cu)]
                               for r in self.cs.reinf_components_with_state])
        try:
            min_index = np.argmin(eps_lo_arr[:, 1])
        except:
            # Returns None in case there is no reinforcement.
            return None
        return eps_lo_arr[min_index][0]

    n_eps = Int(20, auto_set=False, enter_set=True)
    eps_range = Property(depends_on='n_eps,modified')
    @cached_property
    def _get_eps_range(self):
        eps_cu = self.eps_cu
        eps_ccu = 0.8 * eps_cu
        env_reinf = self.strain_env_reinf

        if env_reinf:
            '''Strain envelope for reinforced cross section
            '''
            eps_t_lo = env_reinf.convert_eps_u_2_lo(eps_up=eps_cu)
            eps_t_lo_0 = env_reinf.convert_eps_u_2_lo(eps_up=0.)
            eps_t_u = env_reinf.material_law_.eps_u

            # Strain arrays for the lower rim
            eps_cc_0 = np.linspace(eps_ccu, 0., self.n_eps)
            eps_0_tlo = np.linspace(0., eps_t_lo, self.n_eps)
            eps_tlo_tlo0 = np.linspace(eps_t_lo, eps_t_lo_0, self.n_eps)
            eps_tlo0_tu = np.linspace(eps_t_lo_0, eps_t_u, self.n_eps)

            # Strain arrays for the upper rim
            eps_cc_c = np.linspace(eps_ccu, eps_cu, self.n_eps)
            eps_c_const = self.eps_cu * np.ones_like(eps_cc_0)
            eps_c_0 = np.linspace(eps_cu, 0., self.n_eps)
            eps_0_tu = np.linspace(0., eps_t_u, self.n_eps)

            eps1 = np.vstack([eps_cc_0, eps_cc_c])
            eps2 = np.vstack([eps_0_tlo, eps_c_const])
            eps3 = np.vstack([eps_tlo_tlo0, eps_c_0])
            eps4 = np.vstack([eps_tlo0_tu, eps_0_tu])

            return np.hstack([eps1, eps2, eps3, eps4])
        else:
            '''Strain envelope for cross section without reinforcement
            '''
            # Strain arrays for the lower rim
            eps_cc_0 = np.linspace(eps_ccu, 0., self.n_eps)
            eps_00 = np.zeros_like(eps_cc_0)

            # Strain arrays for the upper rim
            eps_cc_c = np.linspace(eps_ccu, eps_cu, self.n_eps)
            eps_c_0 = np.linspace(eps_cu, 0., self.n_eps)

            eps1 = np.vstack([eps_cc_0, eps_cc_c])
            eps2 = np.vstack([eps_00, eps_c_0])

            return np.hstack([eps1, eps2])

    n_eps_range = Property(depends_on='n_eps,modified')
    @cached_property
    def _get_n_eps_range(self):
        return self.eps_range.shape[1]

    #===========================================================================
    # MN Diagram
    #===========================================================================

    def _get_MN_fn(self, eps_lo, eps_up):
        self.cs.set(eps_lo=eps_lo,
                    eps_up=eps_up)
        return (self.cs.M, self.cs.N)

    MN_vct = Property()
    def _get_MN_vct(self):
        return np.vectorize(self._get_MN_fn)

    MN_arr = Property(depends_on='modified,n_eps')
    @cached_property
    def _get_MN_arr(self):
        return self.MN_vct(self.eps_range[0, :], self.eps_range[1, :])

    #===========================================================================
    # f_eps Diagram
    #===========================================================================

    current_eps_idx = Int(1)  # , auto_set = False, enter_set = True)

    current_eps = Property(depends_on='current_eps_idx,modified')
    @cached_property
    def _get_current_eps(self):
        return self.eps_range[(0, 1), self.current_eps_idx - 1]

    current_MN = Property(depends_on='current_eps_idx,modified')
    @cached_property
    def _get_current_MN(self):
        return self._get_MN_fn(*self.current_eps)

    def plot_eps(self, ax):
        ax.plot(-self.eps_range,
                [0, self.cs.matrix_cs_with_state.geo.height],
                color='black')
        ax.plot(-self.current_eps,
                [0, self.cs.matrix_cs_with_state.geo.height],
                lw=3, color='red')

        ax.spines['left'].set_position('zero')
        ax.spines['right'].set_color('none')
        ax.spines['top'].set_color('none')
        ax.spines['left'].set_smart_bounds(True)
        ax.spines['bottom'].set_smart_bounds(True)
        ax.xaxis.set_ticks_position('bottom')
        ax.yaxis.set_ticks_position('left')

    def plot_MN(self, ax):
        ax.plot(self.MN_arr[0], -self.MN_arr[1], lw=2, color='blue')
        ax.plot(self.current_MN[0], -self.current_MN[1], 'g.',
                markersize=20.0, color='red')

        ax.spines['left'].set_position('zero')
        ax.spines['bottom'].set_position('zero')
        ax.spines['right'].set_color('none')
        ax.spines['top'].set_color('none')
        ax.spines['left'].set_smart_bounds(True)
        ax.spines['bottom'].set_smart_bounds(True)
        ax.xaxis.set_ticks_position('bottom')
        ax.yaxis.set_ticks_position('left')
        ax.grid(b=None, which='major')
        x1, x2, z1, z2 = ax.axis()
        ax.axis([0, x2, z1, z2])

    def plot_custom(self, ax, color='blue', linestyle='-',
                    linewidth=2, label='<unnamed>'):
        M = self.MN_arr[0]
        N = -self.MN_arr[1]
        ax.plot(M, N, lw=linewidth, color=color,
                ls=linestyle, label=label)

        ax.spines['left'].set_position('zero')
        ax.spines['bottom'].set_position('zero')
        ax.spines['right'].set_color('none')
        ax.spines['top'].set_color('none')
        ax.spines['left'].set_smart_bounds(True)
        ax.spines['bottom'].set_smart_bounds(True)
        ax.xaxis.set_ticks_position('bottom')
        ax.yaxis.set_ticks_position('left')
        ax.grid(b=None, which='major')
        x1, x2, z1, z2 = ax.axis()
        z_min = np.min(np.hstack([[z1], N]))
        z_max = np.max(np.hstack([[z2], N]))
        x_max = np.max(np.hstack([[x2], M]))
        ax.axis([0, x_max, z_min, z_max])

    #===========================================================================
    # Visualisation related attributes
    #===========================================================================

    def plot(self, fig):
        ax1 = fig.add_subplot(1, 2, 1)
        self.plot_eps(ax1)
        ax2 = fig.add_subplot(1, 2, 2)
        self.plot_MN(ax2)

    node_name = 'MxN diagram'

    tree_view = View(Group(
                HGroup(
                Group(Item('n_eps', springy=True),
                      Item('current_eps_idx', editor=RangeEditor(low=1,
                                   high_name='n_eps_range',
                                   format='(%s)',
                                   mode='slider',
                                   auto_set=False,
                                   enter_set=False,
                                   ),
                           show_label=False,
                           ),
                      label='Discretization',
                      springy=True
                      ),
                springy=True,
                ),
                scrollable=True,
                ),
                width=1.0,
                height=0.8,
                resizable=True,
                buttons=['OK', 'Cancel'])

if __name__ == '__main__':
    pass



'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Property, cached_property

import numpy as np

from bmcs_beam.mxn.mfn import MFnLineArray

from .matrix_law_base import \
    MatrixLawBase


class MatrixLawLinear(MatrixLawBase):

    '''Effective crack bridge Law based on fiber-bundle-model.'''
    #-----------------------------
    # for bilinear stress-strain-diagram of the concrete (EC2)
    #-----------------------------
    mfn = Property(depends_on='+input')

    @cached_property
    def _get_mfn(self):
        '''bilinear stress-strain-diagram of the concrete
        '''
        # (for standard concrete)
        f_ck = self.f_ck + 8.
        if f_ck <= self.high_strength_level:
            eps_c3 = 0.00175
            eps_cu3 = self.eps_c_u
        # (for high strength concrete)
        else:
            eps_c3 = (1.75 + 0.55 * (f_ck - 50.) / 40.) / 1000.
            eps_cu3 = self.eps_c_u
        # concrete law with limit for eps_c

        xdata = np.hstack([0., eps_c3, eps_cu3])
        ydata = np.hstack([0., (f_ck), (f_ck)])

        return MFnLineArray(xdata=xdata, ydata=ydata)



from bmcs_beam.mxn.matrix_cross_section.matrix_cross_section_geo import \
    MCSGeo

from bmcs_beam.mxn.matrix_cross_section.matrix_cross_section_geo_I import \
    MCSGeoI

from bmcs_beam.mxn.matrix_cross_section.matrix_cross_section_geo_circ import \
    MCSGeoCirc

from bmcs_beam.mxn.matrix_cross_section.matrix_cross_section_geo_rect import \
    MCSGeoRect

from bmcs_beam.mxn.matrix_cross_section.matrix_cross_section import \
    MatrixCrossSection




'''
Created on Sep 4, 2012

@todo: introduce the dock feature for the views
@todo: classify the state changes and provide examples.

@author: rch
'''
from traits.api import \
    Float, Property, cached_property, \
    Instance

from .matrix_cross_section_geo import \
    MCSGeo

from .matrix_cross_section_geo_I import \
    MCSGeoI

from .matrix_cross_section_geo_circ import \
    MCSGeoCirc

from .matrix_cross_section_geo_rect import \
    MCSGeoRect

from traitsui.api import \
    View, Item, Group, HGroup, InstanceEditor

from bmcs_beam.mxn.material_types import \
    MTMatrixMixture

from bmcs_beam.mxn.cross_section_component import \
    CrossSectionComponent

import numpy as np

from bmcs_beam.mxn.utils import \
    KeyRef

STATE_AND_GEOMETRY_CHANGE = 'eps_changed,+geo_input,geo.changed'
STATE_LAW_AND_GEOMETRY_CHANGE = 'eps_changed,+geo_input,geo.changed,material_changed,law_changed,material,material_law'

class MatrixCrossSection(CrossSectionComponent):
    '''Cross section characteristics needed for tensile specimens.
    '''
    n_cj = Float(30, auto_set=False, enter_set=True, geo_input=True)
    '''Number of integration points.
    '''

    material = KeyRef('default_mixture', db=MTMatrixMixture.db)

    x = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Height of the compressive zone
    '''
    @cached_property
    def _get_x(self):
        height = self.geo.height
        eps_lo = self.state.eps_lo
        eps_up = self.state.eps_up
        if eps_up <= 0 and eps_lo <= 0:
            return height
        elif eps_up == eps_lo:
            return 0.
        else:
            return (abs(eps_up) / (abs(eps_up - eps_lo)) * height)

    z_ti_arr = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Discretization of the  compressive zone
    '''
    @cached_property
    def _get_z_ti_arr(self):
        if self.state.eps_up <= 0:  # bending
            zx = min(self.geo.height, self.x)
            return np.linspace(0, zx, self.n_cj)
        elif self.state.eps_lo <= 0:  # bending
            return np.linspace(self.x, self.geo.height, self.n_cj)
        else:  # no compression
            return np.array([0], dtype='f')

    eps_ti_arr = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Compressive strain at each integration layer of the compressive zone [-]:
    '''
    @cached_property
    def _get_eps_ti_arr(self):
        # for calibration us measured compressive strain
        # @todo: use mapped traits instead
        #
        height = self.geo.height
        eps_up = self.state.eps_up
        eps_lo = self.state.eps_lo
        eps_j_arr = (eps_up + (eps_lo - eps_up) * self.z_ti_arr /
                     height)
        return (-np.fabs(eps_j_arr) + eps_j_arr) / 2.0

    zz_ti_arr = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Distance of discrete slices of compressive zone from the bottom
    '''
    @cached_property
    def _get_zz_ti_arr(self):
        return self.geo.height - self.z_ti_arr

    #===========================================================================
    # Cross section geometry and related parameters
    #===========================================================================

    # Implementation remark:
    # Multiple choice value with memory for values that have already been editted.
    # Once a choice is modified to a previously edited geometric objects,
    # its parameters are still available.
    #
    # @todo: is there a possibility to display self-explaining labels of the
    # choice options (they might be a part of the class implementation)
    # This is a candidate for a separate Trait implementation.
    #
    geo = Instance(MCSGeo)
    '''Geometry of the cross section
    '''
    def _geo_default(self):
        return MCSGeoRect(height=0.06, width=0.2)

    geo_lst = Property()
    @cached_property
    def _get_geo_lst(self):
        lst = [MCSGeoRect(), MCSGeoCirc(), MCSGeoI()]
        for i in range(len(lst)):
            if lst[i].__class__ == self.geo.__class__:
                lst[i] = self.geo
        return lst

    w_ti_arr = Property(depends_on=STATE_AND_GEOMETRY_CHANGE)
    '''Discretization of the  compressive zone - weight factors for general cross section
    '''
    @cached_property
    def _get_w_ti_arr(self):
        return self.geo.width_vct(self.geo.height - self.z_ti_arr)

    #===========================================================================
    # Calculation of compressive stresses and forces
    #===========================================================================

    sig_ti_arr = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Stresses at the j-th integration point.
    '''
    @cached_property
    def _get_sig_ti_arr(self):
        return -self.material_law_.mfn_vct(-self.eps_ti_arr)

    f_ti_arr = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Layer force corresponding to the j-th integration point.
    '''
    @cached_property
    def _get_f_ti_arr(self):
        return self.w_ti_arr * self.sig_ti_arr * self.unit_conversion_factor

    N = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Get the resulting normal force.
    '''
    @cached_property
    def _get_N(self):
        return np.trapz(self.f_ti_arr, self.z_ti_arr)

    M = Property(depends_on=STATE_LAW_AND_GEOMETRY_CHANGE)
    '''Get the resulting moment evaluated with respect to the center line
    '''
    @cached_property
    def _get_M(self):
        return np.trapz(self.f_ti_arr * self.z_ti_arr, self.z_ti_arr)

    #===============================================================================
    # Plotting functions
    #===============================================================================

    def plot_eps(self, ax):
        h = self.geo.height

        # eps ti
        ec = np.hstack([self.eps_ti_arr] + [0, 0])
        zz = np.hstack([self.zz_ti_arr] + [0, h ])
        ax.fill(-ec, zz, color='DodgerBlue')

    def plot_sig(self, ax):
        h = self.geo.height

        # sig ti
        ec = np.hstack([self.sig_ti_arr] + [0, 0])
        zz = np.hstack([self.zz_ti_arr] + [0, h ])
        ax.fill(-ec, zz, color='DodgerBlue')

    def plot(self, fig):
        '''Plots the geometry + concrete law
        '''
        ax1 = fig.add_subplot(1, 2, 1)
        self.geo.plot_geometry(ax1)
        ax2 = fig.add_subplot(1, 2, 2)
        self.material_law_.plot_ax(ax2)

    #===========================================================================
    # Auxiliary methods for tree editor
    #===========================================================================
    node_name = 'Matrix cross section'

    tree_view = View(HGroup(
                Group(
                      Item('n_cj'),
                      Item('material'),
                      Item('material_law'),
                      Group(
                      Item('geo', show_label=False,
                           editor=InstanceEditor(name='geo_lst',
                           editable=True), style='custom'),
                      label='Geometry'
                      ),
                      label='Matrix',
                      springy=True
                      ),
                springy=True,
                ),
                width=0.4,
                height=0.4,
                resizable=True,
                buttons=['OK', 'Cancel'])

if __name__ == '__main__':
    mcs = MatrixCrossSection()
    mcs.configure_traits()



'''
Created on 14. 4. 2014

@author: Vancikv

This example shows tree view of the material database
which is the sole default subnode of UseCaseContainer.
'''

from bmcs_beam.mxn.view import \
    MxNTreeView

from bmcs_beam.mxn.use_cases import \
    UseCaseContainer

uc_view = MxNTreeView(root=UseCaseContainer())
uc_view.configure_traits()



'''
Created on Sep 4, 2012

@todo: introduce the dock feature for the views
@todo: classify the state changes and provide examples.


@author: rch
'''
from traits.api import \
    HasStrictTraits, Float

from bmcs_beam.mxn.mxn_tree_node import \
    MxNTreeNode


class CrossSectionState(MxNTreeNode):

    '''
    Cross section state is defined by the linear profile of strains
    with eps_up and eps_lo at the top and at the bottom of the cross section,
    respectively.
    '''

    eps_up = Float(-0.0033, auto_set=False, enter_set=True, eps_input=True)
    eps_lo = Float(0.0140, auto_set=False, enter_set=True, eps_input=True)



#-------------------------------------------------------------------------
#
# Copyright (c) 2009, IMB, RWTH Aachen.
# All rights reserved.
#
# This software is provided without warranty under the terms of the BSD
# license included in simvisage/LICENSE.txt and may be redistributed only
# under the conditions described in the aforementioned license.  The license
# is also available online at http://www.simvisage.com/licenses/BSD.txt
#
# Thanks for using Simvisage open source!
#
# Created on Feb 23, 2010 by: rch

from traits.api import \
    HasTraits, Directory, List, Int, Float, Any, \
    on_trait_change, File, Constant, Instance, Trait, \
    Array, Str, Property, cached_property, WeakRef, \
    Dict, Button, Bool, Enum, Event, implements, \
    DelegatesTo, Expression, Regex, Callable

from traitsui.api import \
    View, Item

# overload the 'get_label' method from 'Item' to display units in the label
from util.traits.ui.item import \
    Item

from numpy import \
    array

from .fabric_layout import \
    FabricLayOut

from matresdev.db.simdb.simdb_class import \
    SimDBClass, SimDBClassExt

from math import \
    cos, sin, pi


class FabricLayUp(SimDBClass):

    '''Describes the type and arrangement of the textile reinforcement
    In the specification of a fabric layup only one type of textile can be used.
    There is an implicit information about the thickness of the fabric layup 
    given by the spacing and the total number of layers. A constant reinforcement
    ratio is assumed (periodic fabric layup).
    '''
    #--------------------------------------------------------------------
    # register a change of the traits with metadata 'input'
    #--------------------------------------------------------------------

    input_change = Event

    @on_trait_change('+input')
    def _set_input_change(self):
        self.input_change = True

    #--------------------------------------------------------------------
    # textile fabric used
    #--------------------------------------------------------------------
    #
    fabric_layout_key = Enum(list(FabricLayOut.db.keys()),
                             simdb=True, input=True, auto_set=False, enter_set=True)

    fabric_layout_ref = Property(
        Instance(SimDBClass), depends_on='fabric_layout_key')

    @cached_property
    def _get_fabric_layout_ref(self):
        return FabricLayOut.db[self.fabric_layout_key]

    #--------------------------------------------------------------------
    # number of layers and spacing
    #--------------------------------------------------------------------

    # total number of layers
    #
    n_layers = Int(1, unit='-', input=True, table_field=True,
                   auto_set=False, enter_set=True)

    # equidistant spacing in thickness direction (z-direction) between the layers
    #
    s_tex_z = Float(1, unit='m', input=True, table_field=True,
                    auto_set=False, enter_set=True)

    #--------------------------------------------------------------------
    # orientation of the layers in the layup:
    #--------------------------------------------------------------------
    # angle with respect to the x-direction of the fabric layup and the
    # 0-degree orientation of the fabric layout. Argument 'n' represents
    # the number of an individual layer. Numbering runs in z-direction
    # from buttom to top starting with '1' (=bottom layer)

    # predefinde functions for the standard fabric layups as Callable
    #

    def get_orientation_90_0(n):
        '''return an angle alternating between 90- and 0-degree
        starting with 90-degree for n = 1 (bottom layer)
        '''
        alpha_degree = abs(sin(pi / 2 * n) * 0. + cos(pi / 2 * n) * 90.)
        return alpha_degree

    def get_orientation_all0(n):
        '''Return an constant angle = 0-degree
        for all layers.
        '''
        return 0.

    def get_orientation_all90(n):
        '''Return an constant angle = 90-degree
        for all layers.
        '''
        return 90.

    orientation_fn_dict = {
        'all0': get_orientation_all0,
        '90_0': get_orientation_90_0,
        'all90': get_orientation_all90,
        'unreinforced': get_orientation_all0
    }

    orientation_fn_key = Enum(list(orientation_fn_dict.keys()),
                              input=True, table_field=False,
                              auto_set=False, enter_set=True)

    orientation_fn_ref = Property(
        Callable, unit='degree', depends_on='input_change')

    @cached_property
    def _get_orientation_fn_ref(self):
        return self.orientation_fn_dict[self.orientation_fn_key]

    #--------------------------------------------------------------------
    # derived properties of the fabric layup
    #--------------------------------------------------------------------

    orientation_list = Property(
        List(Int, unit='degree'), depends_on='input_change')

    @cached_property
    def _get_orientation_list(self):
        return [int(self.orientation_fn_ref(i)) for i in range(self.n_layers)]

    n_layers_0 = Property(Int, unit='-', depends_on='input_change')

    @cached_property
    def _get_n_layers_0(self):
        return self.orientation_list.count(0.)

    n_layers_90 = Property(Int, unit='-', depends_on='input_change')

    @cached_property
    def _get_n_layers_90(self):
        return self.orientation_list.count(90.)

    a_tex = Property(Float, unit='mm^2/m', depends_on='input_change')

    @cached_property
    def _get_a_tex(self):
        '''total cross-sectional-area of the textile reinforcement [mm^2/m]
        for the entire FabricLayUp.
        '''
        a_tex_0 = self.fabric_layout_ref.a_tex_0  # [mm^2/m]
        a_tex_90 = self.fabric_layout_ref.a_tex_90  # [mm^2/m]
        return a_tex_0 * self.n_layers_0 + a_tex_90 * self.n_layers_90

    rho = Property(Float, unit='-', depends_on='input_change')

    @cached_property
    def _get_rho(self):
        '''reinforcement ration of the fabric layup assuming a periodic layup.
        '''
        a_tex = self.a_tex / 1000000.  # [mm^2/m]-->[m^2/m]
        return a_tex / self.thickness  # [-]

    thickness = Property(Float, unit='m', depends_on='input_change')

    @cached_property
    def _get_thickness(self):
        '''thickness of the fabric layup assuming a periodic layup.
        '''
        return self.n_layers * self.s_tex_z  # [m]

    E_tex_arr = Property(Array(Float, unit='MPa'), depends_on='input_change')

    @cached_property
    def _get_E_tex_arr(self):
        '''get the E-modulus of each layer depending on
        its orientation (usually E_tex = E_tex_0 = E_tex_90).
        '''
        return array([self.fabric_layout_ref.get_E_tex(self.orientation_fn_ref(layer_i))
                      for layer_i in range(self.n_layers)])

    E_tex = Property(Float, unit='MPa', depends_on='input_change')

    @cached_property
    def _get_E_tex(self):
        '''get the average E-modulus of all layers, i.e. the 
        entire fabric layup (usually E_tex = E_tex_0 = E_tex_90).
        '''
        return sum(self.E_tex_arr) / self.n_layers

    #------------------------------------------------------------------
    # layup view:
    #------------------------------------------------------------------

    traits_view = View(
        Item('fabric_layout_key'),
        Item('n_layers', format_str="%.0f"),
        Item('s_tex_z', format_str="%.5f"),
        Item('orientation_fn_key'),
        resizable=True,
        scrollable=True,
        height=0.4,
        width=0.5,
    )

#------------------------------------------------------------------

if __name__ == '__main__':
    flu = FabricLayUp(
        n_layers=7,
        s_tex_z=0.030 / (7 + 1),
        orientation_fn_key='all90',
        fabric_layout_key='MAG-07-03'
    )
    flu.configure_traits()



'''
Created on Aug 23, 2012

@author: rch
'''

from bmcs_beam.mxn.constitutive_law import CLBase
from bmcs_beam.mxn.mxn_class_extension import \
    MxNClassExt
from bmcs_beam.mxn.mxn_tree_node import \
    MxNLeafNode
from traits.api import \
    List

from bmcs_beam.mxn.matresdev.db.simdb import \
    SimDBClass, SimDBClassExt


class ReinfLawBase(CLBase, MxNLeafNode, SimDBClass):
    '''Base class for Effective Crack Bridge Laws.'''

    u0 = List([0.0, 0.0])
    node_name = 'Constitutive law'

ReinfLawBase.db = MxNClassExt(
    klass=ReinfLawBase,
    verbose='io',
    node_name='Reinforcement law database'
)

if __name__ == '__main__':
    ReinfLawBase.db.configure_traits()



'''
Created on 26. 2. 2014

@author: Vancikv
'''

from traits.api import \
    Float, Property, cached_property, List, on_trait_change

import numpy as np

from math import exp, log

from .reinf_law_base import \
    ReinfLawBase

class ReinfLawFBM(ReinfLawBase):
    '''Effective crack bridge Law based on fiber-bundle-model.'''

    sig_tex_u = Float(1216., enter_set=True, auto_set=False, input=True)
    eps_u = Float(0.014, enter_set=True, auto_set=False, input=True)
    m = Float(0.5, enter_set=True, auto_set=False, input=True)

    cnames = ['eps_u', 'm']

    u0 = List([0.014, 0.5 ])

    eps_arr = Property(depends_on='eps_u')
    @cached_property
    def _get_eps_arr(self):
        return np.linspace(0, self.eps_u, num=100)

    sig_arr = Property(depends_on='+input')
    @cached_property
    def _get_sig_arr(self):
        sig_tex_u = self.sig_tex_u
        eps_u = self.eps_u
        eps_arr = self.eps_arr
        m = self.m
        return (sig_tex_u / eps_u /
                exp(-pow(exp(-log(m) / m), 1.0 * m)) *
                eps_arr * np.exp(-np.power(eps_arr / eps_u * exp(-log(m) / m), 1.0 * m)))

ReinfLawBase.db.constants['fbm-default'] = ReinfLawFBM(eps_u=0.014)
